import click
from airtrain.integrations.openai.skills import OpenAIChatSkill, OpenAIInput
import os
from dotenv import load_dotenv

load_dotenv()


def initialize_chat():
    return OpenAIChatSkill()


@click.group()
def cli():
    """Airtrain CLI - Your AI Agent Building Assistant"""
    pass


@cli.command()
def chat():
    """Start an interactive chat session with Airtrain"""
    skill = initialize_chat()
    click.echo("Welcome to Airtrain! I'm here to help you build your AI Agent.")
    click.echo("Type 'exit' to end the conversation.\n")

    while True:
        user_input = click.prompt("You", type=str)

        if user_input.lower() == "exit":
            click.echo("\nGoodbye! Have a great day!")
            break

        try:
            input_data = OpenAIInput(
                user_input=user_input,
                system_prompt="You are an AI assistant that helps users build their own AI agents. Be helpful and provide clear explanations.",
                model="gpt-4o",
                temperature=0.7,
            )

            result = skill.process(input_data)
            click.echo(f"\nAirtrain: {result.response}\n")

        except Exception as e:
            click.echo(f"\nError: {str(e)}\n")


def main():
    cli()


if __name__ == "__main__":
    main()



from typing import List, Optional, Dict, Any, Generator
from pydantic import Field
from cerebras.cloud.sdk import Cerebras
from loguru import logger

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import CerebrasCredentials


class CerebrasInput(InputSchema):
    """Schema for Cerebras chat input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )
    model: str = Field(default="llama3.1-8b", description="Cerebras model to use")
    max_tokens: Optional[int] = Field(
        default=1024, description="Maximum tokens in response"
    )
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    stream: bool = Field(
        default=False, description="Whether to stream the response progressively"
    )


class CerebrasOutput(OutputSchema):
    """Schema for Cerebras chat output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(default_factory=dict, description="Usage statistics")


class CerebrasChatSkill(Skill[CerebrasInput, CerebrasOutput]):
    """Skill for Cerebras chat"""

    input_schema = CerebrasInput
    output_schema = CerebrasOutput

    def __init__(self, credentials: Optional[CerebrasCredentials] = None):
        super().__init__()
        self.credentials = credentials or CerebrasCredentials.from_env()
        self.client = Cerebras(
            api_key=self.credentials.cerebras_api_key.get_secret_value()
        )

    def _build_messages(self, input_data: CerebrasInput) -> List[Dict[str, str]]:
        """
        Build messages list from input data including conversation history.

        Args:
            input_data: The input data containing system prompt, conversation history, and user input

        Returns:
            List[Dict[str, str]]: List of messages in the format required by Cerebras
        """
        messages = [{"role": "system", "content": input_data.system_prompt}]

        # Add conversation history if present
        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        # Add current user input
        messages.append({"role": "user", "content": input_data.user_input})

        return messages

    def process_stream(self, input_data: CerebrasInput) -> Generator[str, None, None]:
        """Process the input and stream the response token by token."""
        try:
            messages = self._build_messages(input_data)

            stream = self.client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
                stream=True,
            )

            for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content

        except Exception as e:
            logger.exception(f"Cerebras streaming failed: {str(e)}")
            raise ProcessingError(f"Cerebras streaming failed: {str(e)}")

    def process(self, input_data: CerebrasInput) -> CerebrasOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response = "".join(response_chunks)
                usage = {}  # Usage stats not available in streaming
            else:
                messages = self._build_messages(input_data)
                response = self.client.chat.completions.create(
                    model=input_data.model,
                    messages=messages,
                    temperature=input_data.temperature,
                    max_tokens=input_data.max_tokens,
                )
                usage = (
                    response.usage.model_dump() if hasattr(response, "usage") else {}
                )

            return CerebrasOutput(
                response=response.choices[0].message.content,
                used_model=input_data.model,
                usage=usage,
            )

        except Exception as e:
            logger.exception(f"Cerebras processing failed: {str(e)}")
            raise ProcessingError(f"Cerebras processing failed: {str(e)}")



from typing import List, Optional, Dict, Any, Generator
from pydantic import Field
import requests
from loguru import logger
from openai import OpenAI
from openai.types.chat import ChatCompletionChunk

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import FireworksCredentials
from .models import FireworksMessage, FireworksResponse


class FireworksInput(InputSchema):
    """Schema for Fireworks AI chat input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )
    model: str = Field(
        default="accounts/fireworks/models/deepseek-r1",
        description="Fireworks AI model to use",
    )
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    max_tokens: Optional[int] = Field(
        default=None, description="Maximum tokens in response"
    )
    context_length_exceeded_behavior: str = Field(
        default="truncate", description="Behavior when context length is exceeded"
    )
    stream: bool = Field(
        default=False,
        description="Whether to stream the response token by token",
    )


class FireworksOutput(OutputSchema):
    """Schema for Fireworks AI chat output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, int] = Field(default_factory=dict, description="Usage statistics")


class FireworksChatSkill(Skill[FireworksInput, FireworksOutput]):
    """Skill for interacting with Fireworks AI models"""

    input_schema = FireworksInput
    output_schema = FireworksOutput

    def __init__(self, credentials: Optional[FireworksCredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or FireworksCredentials.from_env()
        self.client = OpenAI(
            base_url="https://api.fireworks.ai/inference/v1",
            api_key=self.credentials.fireworks_api_key.get_secret_value(),
        )

    def _build_messages(self, input_data: FireworksInput) -> List[Dict[str, str]]:
        """Build messages list from input data including conversation history."""
        messages = [{"role": "system", "content": input_data.system_prompt}]

        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        messages.append({"role": "user", "content": input_data.user_input})
        return messages

    def process_stream(self, input_data: FireworksInput) -> Generator[str, None, None]:
        """Process the input and stream the response token by token."""
        try:
            messages = self._build_messages(input_data)

            stream = self.client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
                stream=True,
            )

            for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content

        except Exception as e:
            raise ProcessingError(f"Fireworks streaming failed: {str(e)}")

    def process(self, input_data: FireworksInput) -> FireworksOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                # For streaming, collect the entire response
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response = "".join(response_chunks)
            else:
                # For non-streaming, use regular completion
                messages = self._build_messages(input_data)
                completion = self.client.chat.completions.create(
                    model=input_data.model,
                    messages=messages,
                    temperature=input_data.temperature,
                    max_tokens=input_data.max_tokens,
                    stream=False,
                )
                response = completion.choices[0].message.content

            return FireworksOutput(
                response=response,
                used_model=input_data.model,
                usage={
                    "total_tokens": completion.usage.total_tokens,
                    "prompt_tokens": completion.usage.prompt_tokens,
                    "completion_tokens": completion.usage.completion_tokens,
                },
            )

        except Exception as e:
            raise ProcessingError(f"Fireworks chat failed: {str(e)}")



"""Google Cloud integration module"""

from .credentials import GoogleCloudCredentials
from .skills import GoogleChatSkill
# from .skills import VertexAISkill

__all__ = ["GoogleCloudCredentials", "VertexAISkill"]



from typing import Optional
from together import Together
from airtrain.core.skills import Skill, ProcessingError
from .credentials import TogetherAICredentials
from .schemas import TogetherAIRerankInput, TogetherAIRerankOutput, RerankResult
from .rerank_models_config import get_rerank_model_config


class TogetherAIRerankSkill(Skill[TogetherAIRerankInput, TogetherAIRerankOutput]):
    """Skill for reranking documents using Together AI"""

    input_schema = TogetherAIRerankInput
    output_schema = TogetherAIRerankOutput

    def __init__(self, credentials: Optional[TogetherAICredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or TogetherAICredentials.from_env()
        self.client = Together(
            api_key=self.credentials.together_api_key.get_secret_value()
        )

    def process(self, input_data: TogetherAIRerankInput) -> TogetherAIRerankOutput:
        try:
            # Validate the model exists in our config
            get_rerank_model_config(input_data.model)

            # Call Together AI rerank API
            response = self.client.rerank.create(
                model=input_data.model,
                query=input_data.query,
                documents=input_data.documents,
                top_n=input_data.top_n,
            )

            # Transform results
            results = [
                RerankResult(
                    index=result.index,
                    relevance_score=result.relevance_score,
                    document=input_data.documents[result.index],
                )
                for result in response.results
            ]

            return TogetherAIRerankOutput(results=results, used_model=input_data.model)

        except Exception as e:
            raise ProcessingError(f"Together AI reranking failed: {str(e)}")



from typing import List, Optional, Dict, Any
from pydantic import Field
import google.generativeai as genai
from loguru import logger

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import GeminiCredentials


class GoogleGenerationConfig(InputSchema):
    """Schema for Google generation config"""

    temperature: float = Field(
        default=1.0, description="Temperature for response generation", ge=0, le=1
    )
    top_p: float = Field(
        default=0.95, description="Top p sampling parameter", ge=0, le=1
    )
    top_k: int = Field(default=40, description="Top k sampling parameter")
    max_output_tokens: int = Field(
        default=8192, description="Maximum tokens in response"
    )
    response_mime_type: str = Field(
        default="text/plain", description="Response MIME type"
    )


class GoogleInput(InputSchema):
    """Schema for Google chat input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str | List[Dict[str, str]]]] = Field(
        default_factory=list,
        description="List of conversation messages in Google's format",
    )
    model: str = Field(default="gemini-1.5-flash", description="Google model to use")
    generation_config: GoogleGenerationConfig = Field(
        default_factory=GoogleGenerationConfig, description="Generation configuration"
    )


class GoogleOutput(OutputSchema):
    """Schema for Google chat output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(default_factory=dict, description="Usage statistics")


class GoogleChatSkill(Skill[GoogleInput, GoogleOutput]):
    """Skill for Google chat"""

    input_schema = GoogleInput
    output_schema = GoogleOutput

    def __init__(self, credentials: Optional[GeminiCredentials] = None):
        super().__init__()
        self.credentials = credentials or GeminiCredentials.from_env()
        genai.configure(api_key=self.credentials.gemini_api_key.get_secret_value())

    def _convert_history_format(
        self, history: List[Dict[str, str]]
    ) -> List[Dict[str, List[Dict[str, str]]]]:
        """Convert standard history format to Google's format"""
        google_history = []
        for msg in history:
            google_msg = {
                "role": "user" if msg["role"] == "user" else "model",
                "parts": [{"text": msg["content"]}],
            }
            google_history.append(google_msg)
        return google_history

    def process(self, input_data: GoogleInput) -> GoogleOutput:
        try:
            # Create generation config
            generation_config = {
                "temperature": input_data.generation_config.temperature,
                "top_p": input_data.generation_config.top_p,
                "top_k": input_data.generation_config.top_k,
                "max_output_tokens": input_data.generation_config.max_output_tokens,
                "response_mime_type": input_data.generation_config.response_mime_type,
            }

            # Initialize model
            model = genai.GenerativeModel(
                model_name=input_data.model,
                generation_config=generation_config,
                system_instruction=input_data.system_prompt,
            )

            # Convert history format if needed
            history = (
                input_data.conversation_history
                if input_data.conversation_history
                else self._convert_history_format([])
            )

            # Start chat session
            chat = model.start_chat(history=history)

            # Send message and get response
            response = chat.send_message(input_data.user_input)

            return GoogleOutput(
                response=response.text,
                used_model=input_data.model,
                usage={
                    "prompt_tokens": 0,
                    "completion_tokens": 0,
                    "total_tokens": 0,
                },  # Google API doesn't provide usage stats
            )

        except Exception as e:
            logger.exception(f"Google processing failed: {str(e)}")
            raise ProcessingError(f"Google processing failed: {str(e)}")



"""Airtrain - A platform for building and deploying AI agents with structured skills"""

__version__ = "0.1.33"

# Core imports
from .core.skills import Skill, ProcessingError
from .core.schemas import InputSchema, OutputSchema
from .core.credentials import BaseCredentials

# Integration imports - Credentials
from .integrations.openai.credentials import OpenAICredentials
from .integrations.aws.credentials import AWSCredentials
from .integrations.google.credentials import GoogleCloudCredentials
from .integrations.anthropic.credentials import AnthropicCredentials
from .integrations.groq.credentials import GroqCredentials
from .integrations.together.credentials import TogetherAICredentials
from .integrations.ollama.credentials import OllamaCredentials
from .integrations.sambanova.credentials import SambanovaCredentials
from .integrations.cerebras.credentials import CerebrasCredentials

# Integration imports - Skills
from .integrations.openai.skills import OpenAIChatSkill, OpenAIParserSkill
from .integrations.anthropic.skills import AnthropicChatSkill
from .integrations.aws.skills import AWSBedrockSkill
from .integrations.google.skills import GoogleChatSkill
from .integrations.groq.skills import GroqChatSkill
from .integrations.together.skills import TogetherAIChatSkill
from .integrations.ollama.skills import OllamaChatSkill
from .integrations.sambanova.skills import SambanovaChatSkill
from .integrations.cerebras.skills import CerebrasChatSkill

__all__ = [
    # Core
    "Skill",
    "ProcessingError",
    "InputSchema",
    "OutputSchema",
    "BaseCredentials",
    # Credentials
    "OpenAICredentials",
    "AWSCredentials",
    "GoogleCloudCredentials",
    "AnthropicCredentials",
    "GroqCredentials",
    "TogetherAICredentials",
    "OllamaCredentials",
    "SambanovaCredentials",
    "CerebrasCredentials",
    # Skills
    "OpenAIChatSkill",
    "OpenAIParserSkill",
    "AnthropicChatSkill",
    "AWSBedrockSkill",
    "GoogleChatSkill",
    "GroqChatSkill",
    "TogetherAIChatSkill",
    "OllamaChatSkill",
    "SambanovaChatSkill",
    "CerebrasChatSkill",
]



from .skills import (
    OpenAIChatSkill,
    OpenAIInput,
    OpenAIParserSkill,
    OpenAIOutput,
    OpenAIParserInput,
    OpenAIParserOutput,
)
from .credentials import OpenAICredentials

__all__ = [
    "OpenAIChatSkill",
    "OpenAIInput",
    "OpenAIParserSkill",
    "OpenAIParserInput",
    "OpenAIParserOutput",
    "OpenAICredentials",
    "OpenAIOutput",
]



from typing import List, Dict, Optional
from pydantic import BaseModel, Field
from .skills import FireworksChatSkill, FireworksInput, FireworksOutput

# TODO: Test this thing.


class ConversationState(BaseModel):
    """Model to track conversation state"""

    messages: List[Dict[str, str]] = Field(
        default_factory=list, description="List of conversation messages"
    )
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt for the conversation",
    )
    model: str = Field(
        default="accounts/fireworks/models/deepseek-r1",
        description="Model being used for the conversation",
    )
    temperature: float = Field(default=0.7, description="Temperature setting")
    max_tokens: Optional[int] = Field(default=None, description="Max tokens setting")


class FireworksConversationManager:
    """Manager for handling conversation state with Fireworks AI"""

    def __init__(
        self,
        skill: Optional[FireworksChatSkill] = None,
        system_prompt: str = "You are a helpful assistant.",
        model: str = "accounts/fireworks/models/deepseek-r1",
        temperature: float = 0.7,
        max_tokens: Optional[int] = None,
    ):
        """
        Initialize conversation manager.

        Args:
            skill: FireworksChatSkill instance (creates new one if None)
            system_prompt: Initial system prompt
            model: Model to use
            temperature: Temperature setting
            max_tokens: Max tokens setting
        """
        self.skill = skill or FireworksChatSkill()
        self.state = ConversationState(
            system_prompt=system_prompt,
            model=model,
            temperature=temperature,
            max_tokens=max_tokens,
        )

    def send_message(self, user_input: str) -> FireworksOutput:
        """
        Send a message and get response while maintaining conversation history.

        Args:
            user_input: User's message

        Returns:
            FireworksOutput: Model's response
        """
        # Create input with current conversation state
        input_data = FireworksInput(
            user_input=user_input,
            system_prompt=self.state.system_prompt,
            conversation_history=self.state.messages,
            model=self.state.model,
            temperature=self.state.temperature,
            max_tokens=self.state.max_tokens,
        )

        # Get response
        result = self.skill.process(input_data)

        # Update conversation history
        self.state.messages.extend(
            [
                {"role": "user", "content": user_input},
                {"role": "assistant", "content": result.response},
            ]
        )

        return result

    def reset_conversation(self) -> None:
        """Reset the conversation history while maintaining other settings"""
        self.state.messages = []

    def get_conversation_history(self) -> List[Dict[str, str]]:
        """Get the current conversation history"""
        return self.state.messages.copy()

    def update_system_prompt(self, new_prompt: str) -> None:
        """Update the system prompt for future messages"""
        self.state.system_prompt = new_prompt

    def save_state(self, file_path: str) -> None:
        """Save conversation state to a file"""
        with open(file_path, "w") as f:
            f.write(self.state.model_dump_json(indent=2))

    def load_state(self, file_path: str) -> None:
        """Load conversation state from a file"""
        with open(file_path, "r") as f:
            data = f.read()
            self.state = ConversationState.model_validate_json(data)



from typing import Type, TypeVar, Optional, List, Dict, Any
from pydantic import BaseModel, Field
from openai import OpenAI
from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import FireworksCredentials
import re

# Generic type variable for Pydantic response models
ResponseT = TypeVar("ResponseT", bound=BaseModel)


class FireworksParserInput(InputSchema):
    """Schema for Fireworks structured output input"""

    user_input: str
    system_prompt: str = "You are a helpful assistant that provides structured data."
    model: str = "accounts/fireworks/models/deepseek-r1"
    temperature: float = 0.7
    max_tokens: Optional[int] = None
    response_model: Type[ResponseT]
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )

    class Config:
        arbitrary_types_allowed = True


class FireworksParserOutput(OutputSchema):
    """Schema for Fireworks structured output"""

    parsed_response: BaseModel
    used_model: str
    tokens_used: int
    reasoning: Optional[str] = None


class FireworksParserSkill(Skill[FireworksParserInput, FireworksParserOutput]):
    """Skill for getting structured responses from Fireworks"""

    input_schema = FireworksParserInput
    output_schema = FireworksParserOutput

    def __init__(self, credentials: Optional[FireworksCredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or FireworksCredentials.from_env()
        self.client = OpenAI(
            base_url="https://api.fireworks.ai/inference/v1",
            api_key=self.credentials.fireworks_api_key.get_secret_value(),
        )

    def process(self, input_data: FireworksParserInput) -> FireworksParserOutput:
        try:
            # Build messages list including conversation history
            messages = [{"role": "system", "content": input_data.system_prompt}]

            # Add conversation history if present
            if input_data.conversation_history:
                messages.extend(input_data.conversation_history)

            # Add current user input
            messages.append({"role": "user", "content": input_data.user_input})

            # Make API call with JSON schema
            completion = self.client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                response_format={
                    "type": "json_object",
                    "schema": input_data.response_model.model_json_schema(),
                },
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
            )

            response_content = completion.choices[0].message.content

            # Extract reasoning if present
            reasoning_match = re.search(
                r"<think>(.*?)</think>", response_content, re.DOTALL
            )
            reasoning = reasoning_match.group(1).strip() if reasoning_match else None

            # Extract JSON
            json_match = re.search(r"</think>\s*(\{.*\})", response_content, re.DOTALL)
            json_str = json_match.group(1).strip() if json_match else response_content

            # Parse the response into the specified model
            parsed_response = input_data.response_model.parse_raw(json_str)

            return FireworksParserOutput(
                parsed_response=parsed_response,
                used_model=completion.model,
                tokens_used=completion.usage.total_tokens,
                reasoning=reasoning,
            )

        except Exception as e:
            raise ProcessingError(f"Fireworks parsing failed: {str(e)}")



"""Core modules for Airtrain"""

from .skills import Skill, ProcessingError
from .schemas import InputSchema, OutputSchema
from .credentials import BaseCredentials

__all__ = ["Skill", "ProcessingError", "InputSchema", "OutputSchema", "BaseCredentials"]



from typing import List, Optional
from pydantic import Field, BaseModel
from airtrain.core.schemas import InputSchema, OutputSchema


class RerankResult(BaseModel):
    """Schema for individual rerank result"""

    index: int = Field(..., description="Index of the document in original list")
    relevance_score: float = Field(..., description="Relevance score for the document")
    document: str = Field(..., description="The document content")


class TogetherAIRerankInput(InputSchema):
    """Schema for Together AI rerank input"""

    query: str = Field(..., description="Query to rank documents against")
    documents: List[str] = Field(..., description="List of documents to rank")
    model: str = Field(
        default="Salesforce/Llama-Rank-v1",
        description="Together AI rerank model to use",
    )
    top_n: Optional[int] = Field(
        default=None,
        description="Number of top results to return. If None, returns all results",
    )


class TogetherAIRerankOutput(OutputSchema):
    """Schema for Together AI rerank output"""

    results: List[RerankResult] = Field(..., description="Ranked results")
    used_model: str = Field(..., description="Model used for ranking")



from typing import Dict, NamedTuple


class EmbeddingModelConfig(NamedTuple):
    organization: str
    display_name: str
    model_size: str
    embedding_dimension: int
    context_window: int


TOGETHER_EMBEDDING_MODELS: Dict[str, EmbeddingModelConfig] = {
    "togethercomputer/m2-bert-80M-2k-retrieval": EmbeddingModelConfig(
        organization="Together",
        display_name="M2-BERT-80M-2K-Retrieval",
        model_size="80M",
        embedding_dimension=768,
        context_window=2048,
    ),
    "togethercomputer/m2-bert-80M-8k-retrieval": EmbeddingModelConfig(
        organization="Together",
        display_name="M2-BERT-80M-8K-Retrieval",
        model_size="80M",
        embedding_dimension=768,
        context_window=8192,
    ),
    "togethercomputer/m2-bert-80M-32k-retrieval": EmbeddingModelConfig(
        organization="Together",
        display_name="M2-BERT-80M-32K-Retrieval",
        model_size="80M",
        embedding_dimension=768,
        context_window=32768,
    ),
    "WhereIsAI/UAE-Large-V1": EmbeddingModelConfig(
        organization="WhereIsAI",
        display_name="UAE-Large-v1",
        model_size="326M",
        embedding_dimension=1024,
        context_window=512,
    ),
    "BAAI/bge-large-en-v1.5": EmbeddingModelConfig(
        organization="BAAI",
        display_name="BGE-Large-EN-v1.5",
        model_size="326M",
        embedding_dimension=1024,
        context_window=512,
    ),
    "BAAI/bge-base-en-v1.5": EmbeddingModelConfig(
        organization="BAAI",
        display_name="BGE-Base-EN-v1.5",
        model_size="102M",
        embedding_dimension=768,
        context_window=512,
    ),
    "sentence-transformers/msmarco-bert-base-dot-v5": EmbeddingModelConfig(
        organization="sentence-transformers",
        display_name="Sentence-BERT",
        model_size="110M",
        embedding_dimension=768,
        context_window=512,
    ),
    "bert-base-uncased": EmbeddingModelConfig(
        organization="Hugging Face",
        display_name="BERT",
        model_size="110M",
        embedding_dimension=768,
        context_window=512,
    ),
}


def get_embedding_model_config(model_id: str) -> EmbeddingModelConfig:
    """Get embedding model configuration by model ID"""
    if model_id not in TOGETHER_EMBEDDING_MODELS:
        raise ValueError(f"Model {model_id} not found in Together AI embedding models")
    return TOGETHER_EMBEDDING_MODELS[model_id]


def list_embedding_models_by_organization(
    organization: str,
) -> Dict[str, EmbeddingModelConfig]:
    """Get all embedding models for a specific organization"""
    return {
        model_id: config
        for model_id, config in TOGETHER_EMBEDDING_MODELS.items()
        if config.organization.lower() == organization.lower()
    }


def get_default_embedding_model() -> str:
    """Get the default embedding model ID"""
    return "togethercomputer/m2-bert-80M-32k-retrieval"



from typing import Optional, List, Any
from airtrain.core.skills import Skill, ProcessingError
from airtrain.integrations.openai.skills import OpenAIParserSkill, OpenAIParserInput
from .models import (
    ClothingRecommendation,
    HikingOption,
    InternetAvailability,
    FoodOption,
    PersonalizedRecommendation,
)


class TravelAgentBase(OpenAIParserSkill):
    def __init__(self, credentials=None):
        super().__init__(credentials)
        self.model = "gpt-4o"
        self.temperature = 0.0


class ClothingAgent(TravelAgentBase):
    """Agent for clothing recommendations"""

    def get_recommendations(
        self, location: str, duration: int, activities: List[str], season: str
    ) -> ClothingRecommendation:
        prompt = f"""
        Provide clothing recommendations for a {duration}-day trip to {location} during {season}.
        Activities planned: {', '.join(activities)}.
        Include essential items, weather-specific clothing, and cultural considerations.
        """

        input_data = OpenAIParserInput(
            user_input=prompt,
            system_prompt="You are a travel clothing expert. Provide detailed packing recommendations.",
            response_model=ClothingRecommendation,
            model=self.model,
            temperature=self.temperature,
        )

        result = self.process(input_data)
        return result.parsed_response


class HikingAgent(TravelAgentBase):
    """Agent for hiking recommendations"""

    def get_hiking_options(
        self, location: str, difficulty: str, duration_preference: float
    ) -> List[HikingOption]:
        prompt = f"""
        Find hiking trails in {location} that match:
        - Difficulty level: {difficulty}
        - Preferred duration: around {duration_preference} hours
        Provide detailed trail information and safety tips.
        """

        input_data = OpenAIParserInput(
            user_input=prompt,
            system_prompt="You are a hiking expert. Recommend suitable trails with safety considerations.",
            response_model=List[HikingOption],
            model=self.model,
        )

        result = self.process(input_data)
        return result.parsed_response


class InternetAgent(TravelAgentBase):
    """Agent for internet availability information"""

    def get_connectivity_info(
        self,
        location: str,
        duration: int,
        work_requirements: Optional[List[str]] = None,
    ) -> InternetAvailability:
        prompt = f"""
        Provide detailed internet connectivity information for {location} for a {duration}-day stay.
        {f'Work requirements: {", ".join(work_requirements)}' if work_requirements else ''}
        Include public WiFi spots, recommended providers, and connectivity tips.
        """

        input_data = OpenAIParserInput(
            user_input=prompt,
            system_prompt="You are a connectivity expert. Provide detailed internet availability information.",
            response_model=InternetAvailability,
            model=self.model,
            temperature=self.temperature,
        )

        result = self.process(input_data)
        return result.parsed_response


class FoodAgent(TravelAgentBase):
    """Agent for food recommendations"""

    def get_food_recommendations(
        self,
        location: str,
        dietary_restrictions: Optional[List[str]] = None,
        preferences: Optional[List[str]] = None,
        budget_level: str = "medium",
    ) -> FoodOption:
        prompt = f"""
        Provide food recommendations for {location}.
        {f'Dietary restrictions: {", ".join(dietary_restrictions)}' if dietary_restrictions else ''}
        {f'Food preferences: {", ".join(preferences)}' if preferences else ''}
        Budget level: {budget_level}
        Include local specialties, restaurant recommendations, and food safety tips.
        """

        input_data = OpenAIParserInput(
            user_input=prompt,
            system_prompt="You are a culinary expert. Provide detailed food recommendations.",
            response_model=FoodOption,
            model=self.model,
            temperature=self.temperature,
        )

        result = self.process(input_data)
        return result.parsed_response


class PersonalizedAgent(TravelAgentBase):
    """Agent for personalized travel recommendations"""

    def get_personalized_recommendations(
        self,
        location: str,
        duration: int,
        interests: List[str],
        budget_level: str,
        travel_style: str,
        previous_destinations: Optional[List[str]] = None,
    ) -> PersonalizedRecommendation:
        prompt = f"""
        Create personalized travel recommendations for {location} for {duration} days.
        Interests: {', '.join(interests)}
        Travel style: {travel_style}
        Budget level: {budget_level}
        {f'Previous destinations: {", ".join(previous_destinations)}' if previous_destinations else ''}
        Include hidden gems, local events, and a custom itinerary.
        """

        input_data = OpenAIParserInput(
            user_input=prompt,
            system_prompt="You are a personal travel consultant. Provide tailored recommendations.",
            response_model=PersonalizedRecommendation,
            model=self.model,
            temperature=self.temperature,
        )

        result = self.process(input_data)
        return result.parsed_response


# Similar pattern for other agents...

if __name__ == "__main__":
    # Initialize all agents
    clothing_agent = ClothingAgent()
    hiking_agent = HikingAgent()
    internet_agent = InternetAgent()
    food_agent = FoodAgent()
    personalized_agent = PersonalizedAgent()

    # Example location and common parameters
    location = "Kyoto, Japan"
    duration = 7
    season = "spring"

    try:
        # Get clothing recommendations
        clothing_result = clothing_agent.get_recommendations(
            location=location,
            duration=duration,
            activities=["hiking", "temple visits", "city walking", "photography"],
            season=season,
        )
        print("\n=== Clothing Recommendations ===")
        print(f"Essential items: {', '.join(clothing_result.essential_items)}")
        print(f"Weather specific: {', '.join(clothing_result.weather_specific)}")
        print(
            f"Cultural considerations: {', '.join(clothing_result.cultural_considerations)}"
        )

        # Get hiking options
        hiking_result = hiking_agent.get_hiking_options(
            location=location, difficulty="moderate", duration_preference=4.0
        )
        print("\n=== Hiking Options ===")
        for trail in hiking_result:
            print(f"\nTrail: {trail.trail_name}")
            print(f"Difficulty: {trail.difficulty}")
            print(f"Duration: {trail.duration_hours} hours")
            print(f"Distance: {trail.distance_km} km")

        # Get internet availability
        internet_result = internet_agent.get_connectivity_info(
            location=location,
            duration=duration,
            work_requirements=["video calls", "cloud storage access"],
        )
        print("\n=== Internet Availability ===")
        print(f"General availability: {internet_result.general_availability}")
        print(f"Average speed: {internet_result.average_speed_mbps} Mbps")
        print(
            f"Recommended providers: {', '.join(internet_result.recommended_providers)}"
        )

        # Get food recommendations
        food_result = food_agent.get_food_recommendations(
            location=location,
            dietary_restrictions=["vegetarian"],
            preferences=["traditional", "local specialties"],
            budget_level="medium",
        )
        print("\n=== Food Recommendations ===")
        print("Local specialties:", ", ".join(food_result.local_specialties))
        print("Must-try dishes:", ", ".join(food_result.must_try_dishes))
        for restaurant in food_result.recommended_restaurants[:3]:  # Show top 3
            print(f"Restaurant: {restaurant['name']} - {restaurant['type']}")

        # Get personalized recommendations
        personal_result = personalized_agent.get_personalized_recommendations(
            location=location,
            duration=duration,
            interests=["photography", "culture", "nature", "food"],
            budget_level="medium",
            travel_style="balanced",
            previous_destinations=["Tokyo", "Seoul"],
        )
        print("\n=== Personalized Recommendations ===")
        print("Hidden gems:", ", ".join(personal_result.hidden_gems))
        print("\nCustom Itinerary:")
        for day in personal_result.custom_itinerary:
            print(f"Day {day['day']}: {day['activities']}")

    except ProcessingError as e:
        print(f"Error processing travel recommendations: {str(e)}")
    except Exception as e:
        print(f"Unexpected error: {str(e)}")



from typing import List, Optional, Dict, Any
from pydantic import Field
import boto3
from pathlib import Path
from loguru import logger

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import AWSCredentials


class AWSBedrockInput(InputSchema):
    """Schema for AWS Bedrock chat input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    model: str = Field(
        default="anthropic.claude-3-sonnet-20240229-v1:0",
        description="AWS Bedrock model to use",
    )
    max_tokens: int = Field(default=1024, description="Maximum tokens in response")
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    images: Optional[List[Path]] = Field(
        default=None,
        description="Optional list of image paths to include in the message",
    )


class AWSBedrockOutput(OutputSchema):
    """Schema for AWS Bedrock chat output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(
        default_factory=dict, description="Usage statistics from the API"
    )


class AWSBedrockSkill(Skill[AWSBedrockInput, AWSBedrockOutput]):
    """Skill for interacting with AWS Bedrock models"""

    input_schema = AWSBedrockInput
    output_schema = AWSBedrockOutput

    def __init__(self, credentials: Optional[AWSCredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or AWSCredentials.from_env()
        self.client = boto3.client(
            "bedrock-runtime",
            aws_access_key_id=self.credentials.aws_access_key_id.get_secret_value(),
            aws_secret_access_key=self.credentials.aws_secret_access_key.get_secret_value(),
            region_name=self.credentials.aws_region,
        )

    def process(self, input_data: AWSBedrockInput) -> AWSBedrockOutput:
        """Process the input using AWS Bedrock API"""
        try:
            logger.info(f"Processing request with model {input_data.model}")

            # Prepare request body based on model provider
            if "anthropic" in input_data.model:
                request_body = {
                    "anthropic_version": "bedrock-2023-05-31",
                    "max_tokens": input_data.max_tokens,
                    "temperature": input_data.temperature,
                    "system": input_data.system_prompt,
                    "messages": [{"role": "user", "content": input_data.user_input}],
                }
            else:
                raise ProcessingError(f"Unsupported model: {input_data.model}")

            response = self.client.invoke_model(
                modelId=input_data.model, body=request_body
            )

            # Parse response based on model provider
            if "anthropic" in input_data.model:
                response_data = response["body"]["completion"]
                usage = {
                    "input_tokens": response["body"]["usage"]["input_tokens"],
                    "output_tokens": response["body"]["usage"]["output_tokens"],
                }
            else:
                raise ProcessingError(f"Unsupported model response: {input_data.model}")

            return AWSBedrockOutput(
                response=response_data, used_model=input_data.model, usage=usage
            )

        except Exception as e:
            logger.exception(f"AWS Bedrock processing failed: {str(e)}")
            raise ProcessingError(f"AWS Bedrock processing failed: {str(e)}")



from typing import Optional, Dict, Any
from pydantic import Field
from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import OllamaCredentials


class OllamaInput(InputSchema):
    """Schema for Ollama input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    model: str = Field(default="llama2", description="Ollama model to use")
    max_tokens: int = Field(default=1024, description="Maximum tokens in response")
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )


class OllamaOutput(OutputSchema):
    """Schema for Ollama output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(default_factory=dict, description="Usage statistics")


class OllamaChatSkill(Skill[OllamaInput, OllamaOutput]):
    """Skill for Ollama - Not Implemented"""

    input_schema = OllamaInput
    output_schema = OllamaOutput

    def __init__(self, credentials: Optional[OllamaCredentials] = None):
        raise NotImplementedError("OllamaChatSkill is not implemented yet")

    def process(self, input_data: OllamaInput) -> OllamaOutput:
        raise NotImplementedError("OllamaChatSkill is not implemented yet")



from typing import Type, TypeVar, Optional, List, Dict, Any, Generator
from pydantic import BaseModel, Field
import requests
import json
from loguru import logger

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import FireworksCredentials

ResponseT = TypeVar("ResponseT", bound=BaseModel)


class FireworksStructuredCompletionInput(InputSchema):
    """Schema for Fireworks AI structured completion input"""

    prompt: str = Field(..., description="Input prompt for completion")
    model: str = Field(
        default="accounts/fireworks/models/deepseek-r1",
        description="Fireworks AI model to use",
    )
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    max_tokens: int = Field(default=4096, description="Maximum tokens in response")
    response_model: Type[ResponseT]
    stream: bool = Field(
        default=False,
        description="Whether to stream the response",
    )

    class Config:
        arbitrary_types_allowed = True


class FireworksStructuredCompletionOutput(OutputSchema):
    """Schema for Fireworks AI structured completion output"""

    parsed_response: Any
    used_model: str
    usage: Dict[str, int]


class FireworksStructuredCompletionSkill(
    Skill[FireworksStructuredCompletionInput, FireworksStructuredCompletionOutput]
):
    """Skill for getting structured completion responses from Fireworks AI"""

    input_schema = FireworksStructuredCompletionInput
    output_schema = FireworksStructuredCompletionOutput
    BASE_URL = "https://api.fireworks.ai/inference/v1/completions"

    def __init__(self, credentials: Optional[FireworksCredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or FireworksCredentials.from_env()
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.credentials.fireworks_api_key.get_secret_value()}",
        }

    def _build_payload(
        self, input_data: FireworksStructuredCompletionInput
    ) -> Dict[str, Any]:
        """Build the request payload."""
        return {
            "model": input_data.model,
            "prompt": input_data.prompt,
            "temperature": input_data.temperature,
            "max_tokens": input_data.max_tokens,
            "stream": input_data.stream,
            "response_format": {
                "type": "json_object",
                "schema": {
                    **input_data.response_model.model_json_schema(),
                    "required": [
                        field
                        for field, _ in input_data.response_model.model_fields.items()
                    ],
                },
            },
        }

    def process_stream(
        self, input_data: FireworksStructuredCompletionInput
    ) -> Generator[Dict[str, Any], None, None]:
        """Process the input and stream the response."""
        try:
            payload = self._build_payload(input_data)
            response = requests.post(
                self.BASE_URL,
                headers=self.headers,
                data=json.dumps(payload),
                stream=True,
            )
            response.raise_for_status()

            json_buffer = []
            for line in response.iter_lines():
                if line:
                    try:
                        data = json.loads(line.decode("utf-8").removeprefix("data: "))
                        if data.get("choices") and data["choices"][0].get("text"):
                            content = data["choices"][0]["text"]
                            json_buffer.append(content)
                            yield {"chunk": content}
                    except json.JSONDecodeError:
                        continue

            # Once complete, parse the full JSON
            complete_json = "".join(json_buffer)
            try:
                parsed_response = input_data.response_model.model_validate_json(
                    complete_json
                )
                yield {"complete": parsed_response}
            except Exception as e:
                raise ProcessingError(f"Failed to parse JSON response: {str(e)}")

        except Exception as e:
            raise ProcessingError(f"Fireworks streaming request failed: {str(e)}")

    def process(
        self, input_data: FireworksStructuredCompletionInput
    ) -> FireworksStructuredCompletionOutput:
        """Process the input and return structured response."""
        try:
            if input_data.stream:
                # For streaming, collect and parse the entire response
                json_buffer = []
                parsed_response = None

                for chunk in self.process_stream(input_data):
                    if "chunk" in chunk:
                        json_buffer.append(chunk["chunk"])
                    elif "complete" in chunk:
                        parsed_response = chunk["complete"]

                if parsed_response is None:
                    raise ProcessingError("Failed to parse streamed response")

                return FireworksStructuredCompletionOutput(
                    parsed_response=parsed_response,
                    used_model=input_data.model,
                    usage={},  # Usage stats not available in streaming mode
                )
            else:
                # For non-streaming, use regular request
                payload = self._build_payload(input_data)
                response = requests.post(
                    self.BASE_URL, headers=self.headers, data=json.dumps(payload)
                )
                response.raise_for_status()
                data = response.json()

                response_text = data["choices"][0]["text"]
                parsed_response = input_data.response_model.model_validate_json(
                    response_text
                )

                return FireworksStructuredCompletionOutput(
                    parsed_response=parsed_response,
                    used_model=input_data.model,
                    usage=data["usage"],
                )

        except Exception as e:
            raise ProcessingError(f"Fireworks structured completion failed: {str(e)}")



"""Airtrain contrib package for community contributions"""

from .travel.agents import (
    TravelAgentBase,
    ClothingAgent,
    HikingAgent,
    InternetConnectivityAgent,
    FoodRecommendationAgent,
    PersonalizedRecommendationAgent,
)
from .travel.models import (
    ClothingRecommendation,
    HikingOption,
    InternetAvailability,
    FoodOption,
)

__all__ = [
    "TravelAgentBase",
    "ClothingAgent",
    "HikingAgent",
    "InternetConnectivityAgent",
    "FoodRecommendationAgent",
    "PersonalizedRecommendationAgent",
    "ClothingRecommendation",
    "HikingOption",
    "InternetAvailability",
    "FoodOption",
]



from pydantic import Field, SecretStr, HttpUrl
from airtrain.core.credentials import BaseCredentials, CredentialValidationError


class SambanovaCredentials(BaseCredentials):
    """SambaNova credentials"""

    sambanova_api_key: SecretStr = Field(..., description="SambaNova API key")
    sambanova_endpoint_url: HttpUrl = Field(..., description="SambaNova API endpoint")

    _required_credentials = {"sambanova_api_key", "sambanova_endpoint_url"}

    async def validate_credentials(self) -> bool:
        """Validate SambaNova credentials"""
        try:
            # Implement SambaNova-specific validation
            # This would depend on their API client implementation
            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid SambaNova credentials: {str(e)}")



from typing import List, Optional, Dict, Any
from pydantic import Field, BaseModel


class FireworksMessage(BaseModel):
    """Schema for Fireworks chat message"""

    content: str
    role: str = Field(..., pattern="^(system|user|assistant)$")


class FireworksUsage(BaseModel):
    """Schema for Fireworks API usage statistics"""

    prompt_tokens: int
    completion_tokens: int
    total_tokens: int


class FireworksResponse(BaseModel):
    """Schema for Fireworks API response"""

    id: str
    choices: List[Dict[str, Any]]
    created: int
    model: str
    usage: FireworksUsage



from typing import Dict, NamedTuple


class ModelConfig(NamedTuple):
    organization: str
    display_name: str
    context_length: int
    quantization: str


TOGETHER_MODELS: Dict[str, ModelConfig] = {
    # DeepSeek Models
    "deepseek-ai/DeepSeek-R1": ModelConfig(
        organization="DeepSeek",
        display_name="DeepSeek-R1",
        context_length=32768,
        quantization="FP8",
    ),
    "deepseek-ai/DeepSeek-R1-Distill-Llama-70B": ModelConfig(
        organization="DeepSeek",
        display_name="DeepSeek R1 Distill Llama 70B",
        context_length=131072,
        quantization="FP16",
    ),
    "deepseek-ai/DeepSeek-R1-Distill-Qwen-1.5B": ModelConfig(
        organization="DeepSeek",
        display_name="DeepSeek R1 Distill Qwen 1.5B",
        context_length=131072,
        quantization="FP16",
    ),
    "deepseek-ai/DeepSeek-R1-Distill-Qwen-14B": ModelConfig(
        organization="DeepSeek",
        display_name="DeepSeek R1 Distill Qwen 14B",
        context_length=131072,
        quantization="FP16",
    ),
    "deepseek-ai/DeepSeek-V3": ModelConfig(
        organization="DeepSeek",
        display_name="DeepSeek-V3",
        context_length=16384,
        quantization="FP8",
    ),
    # Meta Models
    "meta-llama/Llama-3.3-70B-Instruct-Turbo": ModelConfig(
        organization="Meta",
        display_name="Llama 3.3 70B Instruct Turbo",
        context_length=131072,
        quantization="FP8",
    ),
    "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo": ModelConfig(
        organization="Meta",
        display_name="Llama 3.1 8B Instruct Turbo",
        context_length=131072,
        quantization="FP8",
    ),
    "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo": ModelConfig(
        organization="Meta",
        display_name="Llama 3.1 70B Instruct Turbo",
        context_length=131072,
        quantization="FP8",
    ),
    "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo": ModelConfig(
        organization="Meta",
        display_name="Llama 3.1 405B Instruct Turbo",
        context_length=130815,
        quantization="FP8",
    ),
    "meta-llama/Meta-Llama-3-8B-Instruct-Turbo": ModelConfig(
        organization="Meta",
        display_name="Llama 3 8B Instruct Turbo",
        context_length=8192,
        quantization="FP8",
    ),
    "meta-llama/Meta-Llama-3-70B-Instruct-Turbo": ModelConfig(
        organization="Meta",
        display_name="Llama 3 70B Instruct Turbo",
        context_length=8192,
        quantization="FP8",
    ),
    "meta-llama/Llama-3.2-3B-Instruct-Turbo": ModelConfig(
        organization="Meta",
        display_name="Llama 3.2 3B Instruct Turbo",
        context_length=131072,
        quantization="FP16",
    ),
    "meta-llama/Meta-Llama-3-8B-Instruct-Lite": ModelConfig(
        organization="Meta",
        display_name="Llama 3 8B Instruct Lite",
        context_length=8192,
        quantization="INT4",
    ),
    "meta-llama/Meta-Llama-3-70B-Instruct-Lite": ModelConfig(
        organization="Meta",
        display_name="Llama 3 70B Instruct Lite",
        context_length=8192,
        quantization="INT4",
    ),
    "meta-llama/Llama-3-8b-chat-hf": ModelConfig(
        organization="Meta",
        display_name="Llama 3 8B Instruct Reference",
        context_length=8192,
        quantization="FP16",
    ),
    "meta-llama/Llama-3-70b-chat-hf": ModelConfig(
        organization="Meta",
        display_name="Llama 3 70B Instruct Reference",
        context_length=8192,
        quantization="FP16",
    ),
    "meta-llama/Llama-2-13b-chat-hf": ModelConfig(
        organization="Meta",
        display_name="LLaMA-2 Chat (13B)",
        context_length=4096,
        quantization="FP16",
    ),
    # Nvidia Models
    "nvidia/Llama-3.1-Nemotron-70B-Instruct-HF": ModelConfig(
        organization="Nvidia",
        display_name="Llama 3.1 Nemotron 70B",
        context_length=32768,
        quantization="FP16",
    ),
    # Qwen Models
    "Qwen/Qwen2.5-Coder-32B-Instruct": ModelConfig(
        organization="Qwen",
        display_name="Qwen 2.5 Coder 32B Instruct",
        context_length=32768,
        quantization="FP16",
    ),
    "Qwen/QwQ-32B-Preview": ModelConfig(
        organization="Qwen",
        display_name="QwQ-32B-Preview",
        context_length=32768,
        quantization="FP16",
    ),
    "Qwen/Qwen2.5-7B-Instruct-Turbo": ModelConfig(
        organization="Qwen",
        display_name="Qwen 2.5 7B Instruct Turbo",
        context_length=32768,
        quantization="FP8",
    ),
    "Qwen/Qwen2.5-72B-Instruct-Turbo": ModelConfig(
        organization="Qwen",
        display_name="Qwen 2.5 72B Instruct Turbo",
        context_length=32768,
        quantization="FP8",
    ),
    "Qwen/Qwen2-72B-Instruct": ModelConfig(
        organization="Qwen",
        display_name="Qwen 2 Instruct (72B)",
        context_length=32768,
        quantization="FP16",
    ),
    "Qwen/Qwen2-VL-72B-Instruct": ModelConfig(
        organization="Qwen",
        display_name="Qwen2 VL 72B Instruct",
        context_length=32768,
        quantization="FP16",
    ),
    # Microsoft Models
    "microsoft/WizardLM-2-8x22B": ModelConfig(
        organization="Microsoft",
        display_name="WizardLM-2 8x22B",
        context_length=65536,
        quantization="FP16",
    ),
    # Google Models
    "google/gemma-2-27b-it": ModelConfig(
        organization="Google",
        display_name="Gemma 2 27B",
        context_length=8192,
        quantization="FP16",
    ),
    "google/gemma-2-9b-it": ModelConfig(
        organization="Google",
        display_name="Gemma 2 9B",
        context_length=8192,
        quantization="FP16",
    ),
    "google/gemma-2b-it": ModelConfig(
        organization="Google",
        display_name="Gemma Instruct (2B)",
        context_length=8192,
        quantization="FP16",
    ),
    # Databricks Models
    "databricks/dbrx-instruct": ModelConfig(
        organization="databricks",
        display_name="DBRX Instruct",
        context_length=32768,
        quantization="FP16",
    ),
    # Gryphe Models
    "Gryphe/MythoMax-L2-13b": ModelConfig(
        organization="Gryphe",
        display_name="MythoMax-L2 (13B)",
        context_length=4096,
        quantization="FP16",
    ),
    # Mistral AI Models
    "mistralai/Mistral-Small-24B-Instruct-2501": ModelConfig(
        organization="mistralai",
        display_name="Mistral Small 3 Instruct (24B)",
        context_length=32768,
        quantization="FP16",
    ),
    "mistralai/Mistral-7B-Instruct-v0.1": ModelConfig(
        organization="mistralai",
        display_name="Mistral (7B) Instruct",
        context_length=8192,
        quantization="FP16",
    ),
    "mistralai/Mistral-7B-Instruct-v0.2": ModelConfig(
        organization="mistralai",
        display_name="Mistral (7B) Instruct v0.2",
        context_length=32768,
        quantization="FP16",
    ),
    "mistralai/Mistral-7B-Instruct-v0.3": ModelConfig(
        organization="mistralai",
        display_name="Mistral (7B) Instruct v0.3",
        context_length=32768,
        quantization="FP16",
    ),
    "mistralai/Mixtral-8x7B-Instruct-v0.1": ModelConfig(
        organization="mistralai",
        display_name="Mixtral-8x7B Instruct (46.7B)",
        context_length=32768,
        quantization="FP16",
    ),
    "mistralai/Mixtral-8x22B-Instruct-v0.1": ModelConfig(
        organization="mistralai",
        display_name="Mixtral-8x22B Instruct (141B)",
        context_length=65536,
        quantization="FP16",
    ),
    # NousResearch Models
    "NousResearch/Nous-Hermes-2-Mixtral-8x7B-DPO": ModelConfig(
        organization="NousResearch",
        display_name="Nous Hermes 2 - Mixtral 8x7B-DPO (46.7B)",
        context_length=32768,
        quantization="FP16",
    ),
    # Upstage Models
    "upstage/SOLAR-10.7B-Instruct-v1.0": ModelConfig(
        organization="upstage",
        display_name="Upstage SOLAR Instruct v1 (11B)",
        context_length=4096,
        quantization="FP16",
    ),
}


def get_model_config(model_id: str) -> ModelConfig:
    """Get model configuration by model ID"""
    if model_id not in TOGETHER_MODELS:
        raise ValueError(f"Model {model_id} not found in Together AI models")
    return TOGETHER_MODELS[model_id]


def list_models_by_organization(organization: str) -> Dict[str, ModelConfig]:
    """Get all models for a specific organization"""
    return {
        model_id: config
        for model_id, config in TOGETHER_MODELS.items()
        if config.organization.lower() == organization.lower()
    }


def get_default_model() -> str:
    """Get the default model ID"""
    return "meta-llama/Llama-3.3-70B-Instruct-Turbo"


if __name__ == "__main__":
    print(len(TOGETHER_MODELS))
    print(get_model_config("meta-llama/Llama-3.3-70B-Instruct-Turbo"))



"""Travel related agents and models"""

from .agents import (
    TravelAgentBase,
    ClothingAgent,
    HikingAgent,
    InternetAgent,
    FoodAgent,
    PersonalizedAgent,
)
from .models import (
    ClothingRecommendation,
    HikingOption,
    InternetAvailability,
    FoodOption,
)
from .agentlib.verification_agent import UserVerificationAgent
from .modellib.verification import UserTravelInfo, TravelCompanion, HealthCondition

__all__ = [
    "TravelAgentBase",
    "ClothingAgent",
    "HikingAgent",
    "InternetConnectivityAgent",
    "FoodRecommendationAgent",
    "PersonalizedRecommendationAgent",
    "ClothingRecommendation",
    "HikingOption",
    "InternetAvailability",
    "FoodOption",
    "UserVerificationAgent",
    "UserTravelInfo",
    "TravelCompanion",
    "HealthCondition",
]



from typing import List, Optional, Dict, Any, Generator, Union
from pydantic import Field
import requests
import json
from loguru import logger

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import FireworksCredentials


class FireworksCompletionInput(InputSchema):
    """Schema for Fireworks AI completion input using requests"""

    prompt: str = Field(..., description="Input prompt for completion")
    model: str = Field(
        default="accounts/fireworks/models/deepseek-r1",
        description="Fireworks AI model to use",
    )
    max_tokens: int = Field(default=4096, description="Maximum tokens in response")
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    top_p: float = Field(
        default=1.0, description="Top p sampling parameter", ge=0, le=1
    )
    top_k: int = Field(default=50, description="Top k sampling parameter", ge=0)
    presence_penalty: float = Field(
        default=0.0, description="Presence penalty", ge=-2.0, le=2.0
    )
    frequency_penalty: float = Field(
        default=0.0, description="Frequency penalty", ge=-2.0, le=2.0
    )
    repetition_penalty: float = Field(
        default=1.0, description="Repetition penalty", ge=0.0
    )
    stop: Optional[Union[str, List[str]]] = Field(
        default=None, description="Stop sequences"
    )
    echo: bool = Field(default=False, description="Echo the prompt in the response")
    stream: bool = Field(default=False, description="Whether to stream the response")


class FireworksCompletionOutput(OutputSchema):
    """Schema for Fireworks AI completion output"""

    response: str
    used_model: str
    usage: Dict[str, int]


class FireworksCompletionSkill(
    Skill[FireworksCompletionInput, FireworksCompletionOutput]
):
    """Skill for text completion using Fireworks AI"""

    input_schema = FireworksCompletionInput
    output_schema = FireworksCompletionOutput
    BASE_URL = "https://api.fireworks.ai/inference/v1/completions"

    def __init__(self, credentials: Optional[FireworksCredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or FireworksCredentials.from_env()
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.credentials.fireworks_api_key.get_secret_value()}",
        }

    def _build_payload(self, input_data: FireworksCompletionInput) -> Dict[str, Any]:
        """Build the request payload."""
        payload = {
            "model": input_data.model,
            "prompt": input_data.prompt,
            "max_tokens": input_data.max_tokens,
            "temperature": input_data.temperature,
            "top_p": input_data.top_p,
            "top_k": input_data.top_k,
            "presence_penalty": input_data.presence_penalty,
            "frequency_penalty": input_data.frequency_penalty,
            "repetition_penalty": input_data.repetition_penalty,
            "echo": input_data.echo,
            "stream": input_data.stream,
        }

        if input_data.stop:
            payload["stop"] = input_data.stop

        return payload

    def process_stream(
        self, input_data: FireworksCompletionInput
    ) -> Generator[str, None, None]:
        """Process the input and stream the response."""
        try:
            payload = self._build_payload(input_data)
            response = requests.post(
                self.BASE_URL,
                headers=self.headers,
                data=json.dumps(payload),
                stream=True,
            )
            response.raise_for_status()

            for line in response.iter_lines():
                if line:
                    try:
                        data = json.loads(line.decode("utf-8").removeprefix("data: "))
                        if data.get("choices") and data["choices"][0].get("text"):
                            yield data["choices"][0]["text"]
                    except json.JSONDecodeError:
                        continue

        except Exception as e:
            raise ProcessingError(f"Fireworks completion streaming failed: {str(e)}")

    def process(
        self, input_data: FireworksCompletionInput
    ) -> FireworksCompletionOutput:
        """Process the input and return completion response."""
        try:
            if input_data.stream:
                # For streaming, collect the entire response
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response_text = "".join(response_chunks)
                usage = {}  # Usage stats not available in streaming mode
            else:
                # For non-streaming, use regular request
                payload = self._build_payload(input_data)
                response = requests.post(
                    self.BASE_URL, headers=self.headers, data=json.dumps(payload)
                )
                response.raise_for_status()
                data = response.json()

                response_text = data["choices"][0]["text"]
                usage = data["usage"]

            return FireworksCompletionOutput(
                response=response_text, used_model=input_data.model, usage=usage
            )

        except Exception as e:
            raise ProcessingError(f"Fireworks completion failed: {str(e)}")



from typing import List, Optional, Dict, Any, Generator, AsyncGenerator
from pydantic import Field
import requests
import json
from loguru import logger
import aiohttp

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import FireworksCredentials


class FireworksRequestInput(InputSchema):
    """Schema for Fireworks AI chat input using requests"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages",
    )
    model: str = Field(
        default="accounts/fireworks/models/deepseek-r1",
        description="Fireworks AI model to use",
    )
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    max_tokens: int = Field(default=4096, description="Maximum tokens in response")
    top_p: float = Field(
        default=1.0, description="Top p sampling parameter", ge=0, le=1
    )
    top_k: int = Field(default=40, description="Top k sampling parameter", ge=0)
    presence_penalty: float = Field(
        default=0.0, description="Presence penalty", ge=-2.0, le=2.0
    )
    frequency_penalty: float = Field(
        default=0.0, description="Frequency penalty", ge=-2.0, le=2.0
    )
    stream: bool = Field(
        default=False,
        description="Whether to stream the response",
    )


class FireworksRequestOutput(OutputSchema):
    """Schema for Fireworks AI chat output"""

    response: str
    used_model: str
    usage: Dict[str, int]


class FireworksRequestSkill(Skill[FireworksRequestInput, FireworksRequestOutput]):
    """Skill for interacting with Fireworks AI models using requests"""

    input_schema = FireworksRequestInput
    output_schema = FireworksRequestOutput
    BASE_URL = "https://api.fireworks.ai/inference/v1/chat/completions"

    def __init__(self, credentials: Optional[FireworksCredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or FireworksCredentials.from_env()
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.credentials.fireworks_api_key.get_secret_value()}",
        }
        self.stream_headers = {
            "Accept": "text/event-stream",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.credentials.fireworks_api_key.get_secret_value()}",
        }

    def _build_messages(
        self, input_data: FireworksRequestInput
    ) -> List[Dict[str, str]]:
        """Build messages list from input data including conversation history."""
        messages = [{"role": "system", "content": input_data.system_prompt}]

        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        messages.append({"role": "user", "content": input_data.user_input})
        return messages

    def _build_payload(self, input_data: FireworksRequestInput) -> Dict[str, Any]:
        """Build the request payload."""
        return {
            "model": input_data.model,
            "messages": self._build_messages(input_data),
            "temperature": input_data.temperature,
            "max_tokens": input_data.max_tokens,
            "top_p": input_data.top_p,
            "top_k": input_data.top_k,
            "presence_penalty": input_data.presence_penalty,
            "frequency_penalty": input_data.frequency_penalty,
            "stream": input_data.stream,
        }

    def process_stream(
        self, input_data: FireworksRequestInput
    ) -> Generator[str, None, None]:
        """Process the input and stream the response."""
        try:
            payload = self._build_payload(input_data)
            response = requests.post(
                self.BASE_URL,
                headers=self.headers,
                data=json.dumps(payload),
                stream=True,
            )
            response.raise_for_status()

            for line in response.iter_lines():
                if line:
                    try:
                        data = json.loads(line.decode("utf-8").removeprefix("data: "))
                        if data["choices"][0]["delta"].get("content"):
                            yield data["choices"][0]["delta"]["content"]
                    except json.JSONDecodeError:
                        continue

        except Exception as e:
            raise ProcessingError(f"Fireworks streaming request failed: {str(e)}")

    def process(self, input_data: FireworksRequestInput) -> FireworksRequestOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                # For streaming, collect the entire response
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response_text = "".join(response_chunks)
                usage = {}  # Usage stats not available in streaming mode
            else:
                # For non-streaming, use regular request
                payload = self._build_payload(input_data)
                response = requests.post(
                    self.BASE_URL, headers=self.headers, data=json.dumps(payload)
                )
                response.raise_for_status()
                data = response.json()

                response_text = data["choices"][0]["message"]["content"]
                usage = data["usage"]

            return FireworksRequestOutput(
                response=response_text, used_model=input_data.model, usage=usage
            )

        except Exception as e:
            raise ProcessingError(f"Fireworks request failed: {str(e)}")

    async def process_async(
        self, input_data: FireworksRequestInput
    ) -> FireworksRequestOutput:
        """Async version of process method using aiohttp"""
        try:
            async with aiohttp.ClientSession() as session:
                payload = self._build_payload(input_data)
                async with session.post(
                    self.BASE_URL, headers=self.headers, json=payload
                ) as response:
                    response.raise_for_status()
                    data = await response.json()

                    return FireworksRequestOutput(
                        response=data["choices"][0]["message"]["content"],
                        used_model=input_data.model,
                        usage=data.get("usage", {}),
                    )

        except Exception as e:
            raise ProcessingError(f"Async Fireworks request failed: {str(e)}")

    async def process_stream_async(
        self, input_data: FireworksRequestInput
    ) -> AsyncGenerator[str, None]:
        """Async version of stream processor using aiohttp"""
        try:
            async with aiohttp.ClientSession() as session:
                payload = self._build_payload(input_data)
                async with session.post(
                    self.BASE_URL, headers=self.stream_headers, json=payload
                ) as response:
                    response.raise_for_status()

                    async for line in response.content:
                        if line.startswith(b"data: "):
                            chunk = json.loads(line[6:].strip())
                            if "choices" in chunk:
                                content = (
                                    chunk["choices"][0]
                                    .get("delta", {})
                                    .get("content", "")
                                )
                                if content:
                                    yield content

        except Exception as e:
            raise ProcessingError(f"Async Fireworks streaming failed: {str(e)}")



from typing import Dict, NamedTuple, Optional
from decimal import Decimal


class OpenAIModelConfig(NamedTuple):
    display_name: str
    base_model: str
    input_price: Decimal
    cached_input_price: Optional[Decimal]
    output_price: Decimal


OPENAI_MODELS: Dict[str, OpenAIModelConfig] = {
    "gpt-4o": OpenAIModelConfig(
        display_name="GPT-4 Optimized",
        base_model="gpt-4o",
        input_price=Decimal("2.50"),
        cached_input_price=Decimal("1.25"),
        output_price=Decimal("10.00"),
    ),
    "gpt-4o-2024-08-06": OpenAIModelConfig(
        display_name="GPT-4 Optimized (2024-08-06)",
        base_model="gpt-4o",
        input_price=Decimal("2.50"),
        cached_input_price=Decimal("1.25"),
        output_price=Decimal("10.00"),
    ),
    "gpt-4o-2024-05-13": OpenAIModelConfig(
        display_name="GPT-4 Optimized (2024-05-13)",
        base_model="gpt-4o",
        input_price=Decimal("5.00"),
        cached_input_price=None,
        output_price=Decimal("15.00"),
    ),
    "gpt-4o-audio-preview-2024-12-17": OpenAIModelConfig(
        display_name="GPT-4 Optimized Audio Preview",
        base_model="gpt-4o-audio-preview",
        input_price=Decimal("2.50"),
        cached_input_price=None,
        output_price=Decimal("10.00"),
    ),
    "gpt-4o-realtime-preview-2024-12-17": OpenAIModelConfig(
        display_name="GPT-4 Optimized Realtime Preview",
        base_model="gpt-4o-realtime-preview",
        input_price=Decimal("5.00"),
        cached_input_price=Decimal("2.50"),
        output_price=Decimal("20.00"),
    ),
    "gpt-4o-mini-2024-07-18": OpenAIModelConfig(
        display_name="GPT-4 Optimized Mini",
        base_model="gpt-4o-mini",
        input_price=Decimal("0.15"),
        cached_input_price=Decimal("0.075"),
        output_price=Decimal("0.60"),
    ),
    "gpt-4o-mini-audio-preview-2024-12-17": OpenAIModelConfig(
        display_name="GPT-4 Optimized Mini Audio Preview",
        base_model="gpt-4o-mini-audio-preview",
        input_price=Decimal("0.15"),
        cached_input_price=None,
        output_price=Decimal("0.60"),
    ),
    "gpt-4o-mini-realtime-preview-2024-12-17": OpenAIModelConfig(
        display_name="GPT-4 Optimized Mini Realtime Preview",
        base_model="gpt-4o-mini-realtime-preview",
        input_price=Decimal("0.60"),
        cached_input_price=Decimal("0.30"),
        output_price=Decimal("2.40"),
    ),
    "o1-2024-12-17": OpenAIModelConfig(
        display_name="O1",
        base_model="o1",
        input_price=Decimal("15.00"),
        cached_input_price=Decimal("7.50"),
        output_price=Decimal("60.00"),
    ),
    "o3-mini-2025-01-31": OpenAIModelConfig(
        display_name="O3 Mini",
        base_model="o3-mini",
        input_price=Decimal("1.10"),
        cached_input_price=Decimal("0.55"),
        output_price=Decimal("4.40"),
    ),
    "o1-mini-2024-09-12": OpenAIModelConfig(
        display_name="O1 Mini",
        base_model="o1-mini",
        input_price=Decimal("1.10"),
        cached_input_price=Decimal("0.55"),
        output_price=Decimal("4.40"),
    ),
}


def get_model_config(model_id: str) -> OpenAIModelConfig:
    """Get model configuration by model ID"""
    if model_id not in OPENAI_MODELS:
        raise ValueError(f"Model {model_id} not found in OpenAI models")
    return OPENAI_MODELS[model_id]


def get_default_model() -> str:
    """Get the default model ID"""
    return "gpt-4o"


def calculate_cost(
    model_id: str, input_tokens: int, output_tokens: int, use_cached: bool = False
) -> Decimal:
    """Calculate cost for token usage"""
    config = get_model_config(model_id)
    input_price = (
        config.cached_input_price
        if (use_cached and config.cached_input_price is not None)
        else config.input_price
    )
    return (
        input_price * Decimal(str(input_tokens))
        + config.output_price * Decimal(str(output_tokens))
    ) / Decimal("1000")



from typing import Any, Dict, Optional, Type, Union, cast, get_args, get_origin
from pydantic import BaseModel, ValidationError, create_model
import json
from uuid import UUID, uuid4


class AirtrainSchema(BaseModel):
    """Base schema class for all Airtrain schemas"""

    _schema_id: Optional[UUID] = None
    _schema_version: str = "1.0.0"

    @classmethod
    def _extract_field_type(cls, field_props: Dict) -> Type:
        """
        Extract Python type from field properties

        Args:
            field_props: Field properties from JSON schema

        Returns:
            Python type for the field
        """
        # Handle direct type specification
        if "type" in field_props:
            return cls._map_json_type_to_python(field_props["type"])

        # Handle anyOf/oneOf cases
        for union_key in ["anyOf", "oneOf"]:
            if union_key in field_props:
                types = []
                for type_option in field_props[union_key]:
                    if "type" in type_option:
                        if type_option["type"] == "null":
                            types.append(type(None))
                        else:
                            types.append(
                                cls._map_json_type_to_python(type_option["type"])
                            )

                # If we have types, create a Union
                if types:
                    return Union[tuple(types)] if len(types) > 1 else types[0]

        # Default to Any if type cannot be determined
        return Any

    @classmethod
    def _get_field_config(cls, field_props: Dict) -> tuple:
        """
        Get field type and default value configuration

        Args:
            field_props: Field properties from JSON schema

        Returns:
            Tuple of (field_type, field_default)
        """
        field_type = cls._extract_field_type(field_props)

        # Handle default values
        if "default" in field_props:
            return (field_type, field_props["default"])

        # Handle Optional/Union types
        if get_origin(field_type) is Union and type(None) in get_args(field_type):
            return (field_type, None)

        # No default value
        return (field_type, ...)

    @classmethod
    def from_json_schema(cls, json_schema: str | Dict) -> "AirtrainSchema":
        """
        Create an AirtrainSchema from a JSON schema

        Args:
            json_schema: JSON schema string or dictionary

        Returns:
            AirtrainSchema instance

        Raises:
            ValidationError: If schema is invalid
        """
        if isinstance(json_schema, str):
            json_schema = json.loads(json_schema)

        # Convert JSON schema to Pydantic model
        assert isinstance(json_schema, dict)
        model_fields = {}
        required_fields = json_schema.get("required", [])

        for field_name, field_props in json_schema["properties"].items():
            field_type, field_default = cls._get_field_config(field_props)

            # Override default for required fields
            if field_name in required_fields:
                field_default = ...

            model_fields[field_name] = (field_type, field_default)

        # Create dynamic model using create_model
        DynamicSchema = create_model("DynamicSchema", __base__=cls, **model_fields)

        return cast(AirtrainSchema, DynamicSchema)

    @classmethod
    def from_pydantic_schema(cls, pydantic_schema: Type[BaseModel]) -> "AirtrainSchema":
        """
        Create an AirtrainSchema from a Pydantic model

        Args:
            pydantic_schema: Pydantic model class

        Returns:
            AirtrainSchema instance
        """
        # Get JSON schema from pydantic model
        schema = pydantic_schema.model_json_schema()

        # Create new schema using from_json_schema
        return cls.from_json_schema(schema)

    @staticmethod
    def _map_json_type_to_python(json_type: str) -> Type:
        """Map JSON schema types to Python types"""
        type_mapping = {
            "string": str,
            "integer": int,
            "number": float,
            "boolean": bool,
            "array": list,
            "object": dict,
        }
        assert json_type in type_mapping, f"Unsupported JSON type: {json_type}"
        return type_mapping[json_type]

    def validate_custom(self) -> None:
        """
        Perform custom validation beyond Pydantic's built-in validation
        To be implemented by subclasses

        Raises:
            ValidationError: If custom validation fails
        """
        pass

    def validate_all(self) -> None:
        """
        Perform all validations including Pydantic and custom

        Raises:
            ValidationError: If any validation fails
        """
        # Pydantic validation happens automatically
        try:
            self.validate_custom()
        except Exception as e:
            raise ValidationError(f"Custom validation failed: {str(e)}")

    def publish(self) -> UUID:
        """
        Publish schema to make it available for use

        Returns:
            UUID: Unique identifier for the published schema
        """
        if not self._schema_id:
            self._schema_id = uuid4()
        # TODO: Implement actual publishing logic
        return self._schema_id

    @classmethod
    def get_by_id(cls, schema_id: UUID) -> "AirtrainSchema":
        """
        Retrieve a published schema by ID

        Args:
            schema_id: UUID of the published schema

        Returns:
            AirtrainSchema instance

        Raises:
            ValueError: If schema not found
        """
        # TODO: Implement schema retrieval logic
        raise NotImplementedError("Schema retrieval not implemented yet")


class InputSchema(AirtrainSchema):
    """Schema for task/skill inputs"""

    def validate_input_specific(self) -> None:
        """
        Perform input-specific validations
        To be implemented by subclasses

        Raises:
            ValidationError: If validation fails
        """
        pass

    def validate_custom(self) -> None:
        """
        Override custom validation to include input-specific validation

        Raises:
            ValidationError: If validation fails
        """
        super().validate_custom()
        self.validate_input_specific()


class OutputSchema(AirtrainSchema):
    """Schema for task/skill outputs"""

    def validate_output_specific(self) -> None:
        """
        Perform output-specific validations
        To be implemented by subclasses

        Raises:
            ValidationError: If validation fails
        """
        pass

    def validate_custom(self) -> None:
        """
        Override custom validation to include output-specific validation

        Raises:
            ValidationError: If validation fails
        """
        super().validate_custom()
        self.validate_output_specific()



"""Anthropic integration for Airtrain"""

from .credentials import AnthropicCredentials
from .skills import AnthropicChatSkill, AnthropicInput, AnthropicOutput

__all__ = [
    "AnthropicCredentials",
    "AnthropicChatSkill",
    "AnthropicInput",
    "AnthropicOutput",
]



from typing import Dict, NamedTuple, Optional


class ImageModelConfig(NamedTuple):
    organization: str
    display_name: str
    default_steps: Optional[int]


TOGETHER_IMAGE_MODELS: Dict[str, ImageModelConfig] = {
    "black-forest-labs/FLUX.1-schnell-Free": ImageModelConfig(
        organization="Black Forest Labs",
        display_name="Flux.1 [schnell] (free)",
        default_steps=None,
    ),
    "black-forest-labs/FLUX.1-schnell": ImageModelConfig(
        organization="Black Forest Labs",
        display_name="Flux.1 [schnell] (Turbo)",
        default_steps=4,
    ),
    "black-forest-labs/FLUX.1-dev": ImageModelConfig(
        organization="Black Forest Labs", display_name="Flux.1 Dev", default_steps=28
    ),
    "black-forest-labs/FLUX.1-canny": ImageModelConfig(
        organization="Black Forest Labs", display_name="Flux.1 Canny", default_steps=28
    ),
    "black-forest-labs/FLUX.1-depth": ImageModelConfig(
        organization="Black Forest Labs", display_name="Flux.1 Depth", default_steps=28
    ),
    "black-forest-labs/FLUX.1-redux": ImageModelConfig(
        organization="Black Forest Labs", display_name="Flux.1 Redux", default_steps=28
    ),
    "black-forest-labs/FLUX.1.1-pro": ImageModelConfig(
        organization="Black Forest Labs",
        display_name="Flux1.1 [pro]",
        default_steps=None,
    ),
    "black-forest-labs/FLUX.1-pro": ImageModelConfig(
        organization="Black Forest Labs",
        display_name="Flux.1 [pro]",
        default_steps=None,
    ),
    "stabilityai/stable-diffusion-xl-base-1.0": ImageModelConfig(
        organization="Stability AI",
        display_name="Stable Diffusion XL 1.0",
        default_steps=None,
    ),
}


def get_image_model_config(model_id: str) -> ImageModelConfig:
    """Get image model configuration by model ID"""
    if model_id not in TOGETHER_IMAGE_MODELS:
        raise ValueError(f"Model {model_id} not found in Together AI image models")
    return TOGETHER_IMAGE_MODELS[model_id]


def list_image_models_by_organization(organization: str) -> Dict[str, ImageModelConfig]:
    """Get all image models for a specific organization"""
    return {
        model_id: config
        for model_id, config in TOGETHER_IMAGE_MODELS.items()
        if config.organization.lower() == organization.lower()
    }


def get_default_image_model() -> str:
    """Get the default image model ID"""
    return "black-forest-labs/FLUX.1-schnell-Free"



from typing import Type, TypeVar, Optional, List, Dict, Any, Generator
from pydantic import BaseModel, Field
import requests
import json
from loguru import logger
import re

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import FireworksCredentials

ResponseT = TypeVar("ResponseT", bound=BaseModel)


class FireworksStructuredRequestInput(InputSchema):
    """Schema for Fireworks AI structured output input using requests"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant that provides structured data.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages",
    )
    model: str = Field(
        default="accounts/fireworks/models/deepseek-r1",
        description="Fireworks AI model to use",
    )
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    max_tokens: int = Field(default=4096, description="Maximum tokens in response")
    response_model: Type[ResponseT]
    stream: bool = Field(
        default=False,
        description="Whether to stream the response",
    )

    class Config:
        arbitrary_types_allowed = True


class FireworksStructuredRequestOutput(OutputSchema):
    """Schema for Fireworks AI structured output"""

    parsed_response: Any
    used_model: str
    usage: Dict[str, int]
    reasoning: Optional[str] = None


class FireworksStructuredRequestSkill(
    Skill[FireworksStructuredRequestInput, FireworksStructuredRequestOutput]
):
    """Skill for getting structured responses from Fireworks AI using requests"""

    input_schema = FireworksStructuredRequestInput
    output_schema = FireworksStructuredRequestOutput
    BASE_URL = "https://api.fireworks.ai/inference/v1/chat/completions"

    def __init__(self, credentials: Optional[FireworksCredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or FireworksCredentials.from_env()
        self.headers = {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.credentials.fireworks_api_key.get_secret_value()}",
        }

    def _build_messages(
        self, input_data: FireworksStructuredRequestInput
    ) -> List[Dict[str, str]]:
        """Build messages list from input data including conversation history."""
        messages = [{"role": "system", "content": input_data.system_prompt}]

        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        messages.append({"role": "user", "content": input_data.user_input})
        return messages

    def _build_payload(
        self, input_data: FireworksStructuredRequestInput
    ) -> Dict[str, Any]:
        """Build the request payload."""
        return {
            "model": input_data.model,
            "messages": self._build_messages(input_data),
            "temperature": input_data.temperature,
            "max_tokens": input_data.max_tokens,
            "stream": input_data.stream,
            "response_format": {
                "type": "json_object",
                "schema": {
                    **input_data.response_model.model_json_schema(),
                    "required": [
                        field
                        for field, _ in input_data.response_model.model_fields.items()
                    ],
                },
            },
        }

    def process_stream(
        self, input_data: FireworksStructuredRequestInput
    ) -> Generator[Dict[str, Any], None, None]:
        """Process the input and stream the response."""
        try:
            payload = self._build_payload(input_data)
            response = requests.post(
                self.BASE_URL,
                headers=self.headers,
                data=json.dumps(payload),
                stream=True,
            )
            response.raise_for_status()

            json_buffer = []
            for line in response.iter_lines():
                if line:
                    try:
                        data = json.loads(line.decode("utf-8").removeprefix("data: "))
                        if data["choices"][0]["delta"].get("content"):
                            content = data["choices"][0]["delta"]["content"]
                            json_buffer.append(content)
                            yield {"chunk": content}
                    except json.JSONDecodeError:
                        continue

            # Once complete, parse the full response with think tags
            complete_response = "".join(json_buffer)
            reasoning, json_str = self._parse_response_content(complete_response)

            try:
                parsed_response = input_data.response_model.model_validate_json(
                    json_str
                )
                yield {"complete": parsed_response, "reasoning": reasoning}
            except Exception as e:
                raise ProcessingError(f"Failed to parse JSON response: {str(e)}")

        except Exception as e:
            raise ProcessingError(f"Fireworks streaming request failed: {str(e)}")

    def _parse_response_content(self, content: str) -> tuple[Optional[str], str]:
        """Parse response content to extract reasoning and JSON."""
        # Extract reasoning if present
        reasoning_match = re.search(r"<think>(.*?)</think>", content, re.DOTALL)
        reasoning = reasoning_match.group(1).strip() if reasoning_match else None

        # Extract JSON
        json_match = re.search(r"</think>\s*(\{.*\})", content, re.DOTALL)
        json_str = json_match.group(1).strip() if json_match else content

        return reasoning, json_str

    def process(
        self, input_data: FireworksStructuredRequestInput
    ) -> FireworksStructuredRequestOutput:
        """Process the input and return structured response."""
        try:
            if input_data.stream:
                # For streaming, collect and parse the entire response
                json_buffer = []
                parsed_response = None
                reasoning = None

                for chunk in self.process_stream(input_data):
                    if "chunk" in chunk:
                        json_buffer.append(chunk["chunk"])
                    elif "complete" in chunk:
                        parsed_response = chunk["complete"]
                        reasoning = chunk.get("reasoning")

                if parsed_response is None:
                    raise ProcessingError("Failed to parse streamed response")

                return FireworksStructuredRequestOutput(
                    parsed_response=parsed_response,
                    used_model=input_data.model,
                    usage={},  # Usage stats not available in streaming mode
                    reasoning=reasoning,
                )
            else:
                # For non-streaming, use regular request
                payload = self._build_payload(input_data)
                response = requests.post(
                    self.BASE_URL, headers=self.headers, data=json.dumps(payload)
                )
                response.raise_for_status()
                data = response.json()

                response_content = data["choices"][0]["message"]["content"]

                # Parse the response content to extract reasoning and JSON
                reasoning, json_str = self._parse_response_content(response_content)

                # Parse the JSON string into the specified model
                parsed_response = input_data.response_model.model_validate_json(
                    json_str
                )

                return FireworksStructuredRequestOutput(
                    parsed_response=parsed_response,
                    used_model=input_data.model,
                    usage=data["usage"],
                    reasoning=reasoning,  # Add reasoning to output if present
                )

        except Exception as e:
            raise ProcessingError(f"Fireworks structured request failed: {str(e)}")



from pydantic import SecretStr, BaseModel
from typing import Optional
import os


class FireworksCredentials(BaseModel):
    """Credentials for Fireworks AI API"""

    fireworks_api_key: SecretStr

    @classmethod
    def from_env(cls) -> "FireworksCredentials":
        """Create credentials from environment variables"""
        api_key = os.getenv("FIREWORKS_API_KEY")
        if not api_key:
            raise ValueError("FIREWORKS_API_KEY environment variable not set")

        return cls(fireworks_api_key=api_key)



from typing import Dict, NamedTuple


class RerankModelConfig(NamedTuple):
    organization: str
    display_name: str
    model_size: str
    max_doc_size: int
    max_docs: int


TOGETHER_RERANK_MODELS: Dict[str, RerankModelConfig] = {
    "Salesforce/Llama-Rank-v1": RerankModelConfig(
        organization="Salesforce",
        display_name="LlamaRank",
        model_size="8B",
        max_doc_size=8192,
        max_docs=1024,
    )
}


def get_rerank_model_config(model_id: str) -> RerankModelConfig:
    """Get rerank model configuration by model ID"""
    if model_id not in TOGETHER_RERANK_MODELS:
        raise ValueError(f"Model {model_id} not found in Together AI rerank models")
    return TOGETHER_RERANK_MODELS[model_id]


def list_rerank_models_by_organization(
    organization: str,
) -> Dict[str, RerankModelConfig]:
    """Get all rerank models for a specific organization"""
    return {
        model_id: config
        for model_id, config in TOGETHER_RERANK_MODELS.items()
        if config.organization.lower() == organization.lower()
    }


def get_default_rerank_model() -> str:
    """Get the default rerank model ID"""
    return "Salesforce/Llama-Rank-v1"



from typing import List, Optional, Dict, Any
from pydantic import BaseModel, Field, validator


class ClothingRecommendation(BaseModel):
    """Model for clothing recommendations"""

    essential_items: List[str]
    weather_specific: List[str]
    activity_specific: List[str]
    cultural_considerations: List[str]
    packing_tips: List[str]


class HikingOption(BaseModel):
    """Model for hiking recommendations"""

    trail_name: str
    difficulty: str
    duration_hours: float
    distance_km: float
    elevation_gain_m: float
    best_season: List[str]
    required_gear: List[str]
    safety_tips: List[str]
    highlights: List[str]


class InternetAvailability(BaseModel):
    """Model for internet availability information"""

    general_availability: str
    average_speed_mbps: float
    public_wifi_spots: List[str]
    recommended_providers: List[str]
    connectivity_tips: List[str]
    offline_alternatives: List[str]


class FoodOption(BaseModel):
    """Model for food recommendations"""

    local_specialties: List[str]
    recommended_restaurants: List[Dict[str, str]]
    dietary_considerations: List[str]
    food_safety_tips: List[str]
    price_ranges: Dict[str, str]
    must_try_dishes: List[str]


class PersonalizedRecommendation(BaseModel):
    """Model for personalized recommendations"""

    activities: List[Dict[str, str]]
    hidden_gems: List[str]
    local_events: List[Dict[str, str]]
    custom_itinerary: List[Dict[str, Any]]
    safety_tips: List[str]
    budget_recommendations: Dict[str, str]



from typing import Optional, List
from pathlib import Path
from pydantic import Field
from together import Together
import base64
import time

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import TogetherAICredentials
from .image_models_config import get_image_model_config, get_default_image_model


class TogetherAIImageInput(InputSchema):
    """Schema for Together AI image generation input"""

    prompt: str = Field(..., description="Text prompt for image generation")
    model: str = Field(
        default=get_default_image_model(), description="Together AI image model to use"
    )
    steps: int = Field(default=10, description="Number of inference steps", ge=1, le=50)
    n: int = Field(default=1, description="Number of images to generate", ge=1, le=4)
    size: str = Field(
        default="1024x1024", description="Image size in format WIDTHxHEIGHT"
    )
    negative_prompt: Optional[str] = Field(
        default=None, description="Things to exclude from the generation"
    )
    seed: Optional[int] = Field(
        default=None, description="Random seed for reproducibility"
    )


class GeneratedImage(OutputSchema):
    """Individual generated image data"""

    b64_json: Optional[str] = Field(None, description="Base64 encoded image data")
    url: str = Field(..., description="URL of the generated image")
    seed: Optional[int] = Field(None, description="Seed used for this image")
    finish_reason: Optional[str] = Field(
        None, description="Reason for finishing generation"
    )


class TogetherAIImageOutput(OutputSchema):
    """Schema for Together AI image generation output"""

    images: List[GeneratedImage] = Field(..., description="List of generated images")
    model: str = Field(..., description="Model used for generation")
    prompt: str = Field(..., description="Original prompt used")
    total_time: float = Field(..., description="Time taken for generation in seconds")
    usage: dict = Field(default_factory=dict, description="Usage statistics")


class TogetherAIImageSkill(Skill[TogetherAIImageInput, TogetherAIImageOutput]):
    """Skill for generating images using Together AI"""

    input_schema = TogetherAIImageInput
    output_schema = TogetherAIImageOutput

    def __init__(self, credentials: Optional[TogetherAICredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or TogetherAICredentials.from_env()
        self.client = Together(
            api_key=self.credentials.together_api_key.get_secret_value()
        )

    def process(self, input_data: TogetherAIImageInput) -> TogetherAIImageOutput:
        try:
            # Validate the model exists in our config
            get_image_model_config(input_data.model)

            start_time = time.time()

            # Generate images
            response = self.client.images.generate(
                prompt=input_data.prompt,
                model=input_data.model,
                steps=input_data.steps,
                n=input_data.n,
                size=input_data.size,
                negative_prompt=input_data.negative_prompt,
                seed=input_data.seed,
            )

            # Calculate total time
            total_time = time.time() - start_time

            # Convert response to our output format
            generated_images = []
            for img in response.data:
                if not hasattr(img, "url"):
                    raise ProcessingError(
                        f"No URL found in API response. Response structure: {dir(img)}"
                    )

                generated_images.append(
                    GeneratedImage(
                        url=img.url,
                        seed=getattr(img, "seed", None),
                        finish_reason=getattr(img, "finish_reason", None),
                    )
                )

            return TogetherAIImageOutput(
                images=generated_images,
                model=input_data.model,
                prompt=input_data.prompt,
                total_time=total_time,
                usage=getattr(response, "usage", {}),
            )

        except Exception as e:
            raise ProcessingError(f"Together AI image generation failed: {str(e)}")

    def save_images(
        self, output: TogetherAIImageOutput, output_dir: Path
    ) -> List[Path]:
        """
        Save generated images to disk

        Args:
            output (TogetherAIImageOutput): Generation output containing images
            output_dir (Path): Directory to save images

        Returns:
            List[Path]: List of paths to saved images
        """
        output_dir = Path(output_dir)
        output_dir.mkdir(parents=True, exist_ok=True)

        saved_paths = []
        for i, img in enumerate(output.images):
            output_path = output_dir / f"image_{i}.png"
            image_data = base64.b64decode(img.b64_json)

            with open(output_path, "wb") as f:
                f.write(image_data)

            saved_paths.append(output_path)

        return saved_paths



from pydantic import Field, SecretStr
from airtrain.core.credentials import BaseCredentials, CredentialValidationError
from groq import Groq


class GroqCredentials(BaseCredentials):
    """Groq API credentials"""

    groq_api_key: SecretStr = Field(..., description="Groq API key")

    _required_credentials = {"groq_api_key"}

    async def validate_credentials(self) -> bool:
        """Validate Groq credentials"""
        try:
            client = Groq(api_key=self.groq_api_key.get_secret_value())
            await client.chat.completions.create(
                messages=[{"role": "user", "content": "Hi"}],
                model="mixtral-8x7b-32768",
                max_tokens=1,
            )
            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid Groq credentials: {str(e)}")



from pydantic import Field, SecretStr
from airtrain.core.credentials import BaseCredentials, CredentialValidationError
import together


class TogetherAICredentials(BaseCredentials):
    """Together AI credentials"""

    together_api_key: SecretStr = Field(..., description="Together AI API key")

    _required_credentials = {"together_api_key"}

    async def validate_credentials(self) -> bool:
        """Validate Together AI credentials"""
        try:
            together.api_key = self.together_api_key.get_secret_value()
            await together.Models.list()
            return True
        except Exception as e:
            raise CredentialValidationError(
                f"Invalid Together AI credentials: {str(e)}"
            )



from typing import List, Optional
from pydantic import BaseModel, Field, validator
from pathlib import Path


class TogetherAIImageInput(BaseModel):
    """Schema for Together AI image generation input"""

    prompt: str = Field(..., description="Text prompt for image generation")
    model: str = Field(
        default="black-forest-labs/FLUX.1-schnell-Free",
        description="Together AI image model to use",
    )
    steps: int = Field(default=10, description="Number of inference steps", ge=1, le=50)
    n: int = Field(default=1, description="Number of images to generate", ge=1, le=4)
    size: str = Field(
        default="1024x1024", description="Image size in format WIDTHxHEIGHT"
    )
    negative_prompt: Optional[str] = Field(
        default=None, description="Things to exclude from the generation"
    )
    seed: Optional[int] = Field(
        default=None, description="Random seed for reproducibility"
    )

    @validator("size")
    def validate_size(cls, v):
        try:
            width, height = map(int, v.split("x"))
            if width <= 0 or height <= 0:
                raise ValueError
            return v
        except:
            raise ValueError("Size must be in format WIDTHxHEIGHT (e.g., 1024x1024)")


class GeneratedImage(BaseModel):
    """Individual generated image data"""

    b64_json: str = Field(..., description="Base64 encoded image data")
    seed: Optional[int] = Field(None, description="Seed used for this image")
    finish_reason: Optional[str] = Field(
        None, description="Reason for finishing generation"
    )


class TogetherAIImageOutput(BaseModel):
    """Schema for Together AI image generation output"""

    images: List[GeneratedImage] = Field(..., description="List of generated images")
    model: str = Field(..., description="Model used for generation")
    prompt: str = Field(..., description="Original prompt used")
    total_time: float = Field(..., description="Time taken for generation in seconds")
    usage: dict = Field(
        default_factory=dict, description="Usage statistics and billing information"
    )



from typing import Optional
from pydantic import Field, SecretStr
from airtrain.core.credentials import BaseCredentials, CredentialValidationError
import boto3


class AWSCredentials(BaseCredentials):
    """AWS credentials"""

    aws_access_key_id: SecretStr = Field(..., description="AWS Access Key ID")
    aws_secret_access_key: SecretStr = Field(..., description="AWS Secret Access Key")
    aws_region: str = Field(default="us-east-1", description="AWS Region")
    aws_session_token: Optional[SecretStr] = Field(
        None, description="AWS Session Token"
    )

    _required_credentials = {"aws_access_key_id", "aws_secret_access_key"}

    async def validate_credentials(self) -> bool:
        """Validate AWS credentials by making a test API call"""
        try:
            session = boto3.Session(
                aws_access_key_id=self.aws_access_key_id.get_secret_value(),
                aws_secret_access_key=self.aws_secret_access_key.get_secret_value(),
                aws_session_token=(
                    self.aws_session_token.get_secret_value()
                    if self.aws_session_token
                    else None
                ),
                region_name=self.aws_region,
            )
            sts = session.client("sts")
            sts.get_caller_identity()
            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid AWS credentials: {str(e)}")



from typing import Optional, Dict, Any, List, Generator
from pydantic import Field
from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import SambanovaCredentials
import openai


class SambanovaInput(InputSchema):
    """Schema for Sambanova input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )
    model: str = Field(
        default="DeepSeek-R1-Distill-Llama-70B", description="Sambanova model to use"
    )
    max_tokens: int = Field(default=1024, description="Maximum tokens in response")
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    top_p: float = Field(
        default=0.1, description="Top p sampling parameter", ge=0, le=1
    )
    stream: bool = Field(
        default=False, description="Whether to stream the response progressively"
    )


class SambanovaOutput(OutputSchema):
    """Schema for Sambanova output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(default_factory=dict, description="Usage statistics")


class SambanovaChatSkill(Skill[SambanovaInput, SambanovaOutput]):
    """Skill for Sambanova chat"""

    input_schema = SambanovaInput
    output_schema = SambanovaOutput

    def __init__(self, credentials: Optional[SambanovaCredentials] = None):
        super().__init__()
        self.credentials = credentials or SambanovaCredentials.from_env()
        self.client = openai.OpenAI(
            api_key=self.credentials.sambanova_api_key.get_secret_value(),
            base_url="https://api.sambanova.ai/v1",
        )

    def _build_messages(self, input_data: SambanovaInput) -> List[Dict[str, str]]:
        """
        Build messages list from input data including conversation history.

        Args:
            input_data: The input data containing system prompt, conversation history, and user input

        Returns:
            List[Dict[str, str]]: List of messages in the format required by Sambanova
        """
        messages = [{"role": "system", "content": input_data.system_prompt}]

        # Add conversation history if present
        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        # Add current user input
        messages.append({"role": "user", "content": input_data.user_input})

        return messages

    def process_stream(self, input_data: SambanovaInput) -> Generator[str, None, None]:
        """Process the input and stream the response token by token."""
        try:
            messages = self._build_messages(input_data)

            stream = self.client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
                top_p=input_data.top_p,
                stream=True,
            )

            for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content

        except Exception as e:
            raise ProcessingError(f"Sambanova streaming failed: {str(e)}")

    def process(self, input_data: SambanovaInput) -> SambanovaOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response = "".join(response_chunks)
                usage = {}  # Usage stats not available in streaming
            else:
                messages = self._build_messages(input_data)
                response = self.client.chat.completions.create(
                    model=input_data.model,
                    messages=messages,
                    temperature=input_data.temperature,
                    max_tokens=input_data.max_tokens,
                    top_p=input_data.top_p,
                )
                usage = (
                    response.usage.model_dump() if hasattr(response, "usage") else {}
                )

            return SambanovaOutput(
                response=response.choices[0].message.content,
                used_model=input_data.model,
                usage=usage,
            )

        except Exception as e:
            raise ProcessingError(f"Sambanova processing failed: {str(e)}")



from typing import Dict, NamedTuple


class VisionModelConfig(NamedTuple):
    organization: str
    display_name: str
    context_length: int


TOGETHER_VISION_MODELS: Dict[str, VisionModelConfig] = {
    "meta-llama/Llama-Vision-Free": VisionModelConfig(
        organization="Meta",
        display_name="(Free) Llama 3.2 11B Vision Instruct Turbo",
        context_length=131072,
    ),
    "meta-llama/Llama-3.2-11B-Vision-Instruct-Turbo": VisionModelConfig(
        organization="Meta",
        display_name="Llama 3.2 11B Vision Instruct Turbo",
        context_length=131072,
    ),
    "meta-llama/Llama-3.2-90B-Vision-Instruct-Turbo": VisionModelConfig(
        organization="Meta",
        display_name="Llama 3.2 90B Vision Instruct Turbo",
        context_length=131072,
    ),
}


def get_vision_model_config(model_id: str) -> VisionModelConfig:
    """Get vision model configuration by model ID"""
    if model_id not in TOGETHER_VISION_MODELS:
        raise ValueError(f"Model {model_id} not found in Together AI vision models")
    return TOGETHER_VISION_MODELS[model_id]


def list_vision_models_by_organization(
    organization: str,
) -> Dict[str, VisionModelConfig]:
    """Get all vision models for a specific organization"""
    return {
        model_id: config
        for model_id, config in TOGETHER_VISION_MODELS.items()
        if config.organization.lower() == organization.lower()
    }


def get_default_vision_model() -> str:
    """Get the default vision model ID"""
    return "meta-llama/Llama-Vision-Free"



from pydantic import Field, SecretStr, validator
from airtrain.core.credentials import BaseCredentials, CredentialValidationError
from anthropic import Anthropic


class AnthropicCredentials(BaseCredentials):
    """Anthropic API credentials"""

    anthropic_api_key: SecretStr = Field(..., description="Anthropic API key")
    version: str = Field(default="2023-06-01", description="API Version")

    _required_credentials = {"anthropic_api_key"}

    @validator("anthropic_api_key")
    def validate_api_key_format(cls, v: SecretStr) -> SecretStr:
        key = v.get_secret_value()
        if not key.startswith("sk-ant-"):
            raise ValueError("Anthropic API key must start with 'sk-ant-'")
        return v

    async def validate_credentials(self) -> bool:
        """Validate Anthropic credentials"""
        try:
            client = Anthropic(api_key=self.anthropic_api_key.get_secret_value())
            client.messages.create(
                model="claude-3-opus-20240229",
                max_tokens=1,
                messages=[{"role": "user", "content": "Hi"}],
            )
            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid Anthropic credentials: {str(e)}")



from typing import AsyncGenerator, List, Optional, Dict, TypeVar, Type, Generator
from pydantic import Field, BaseModel
from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletionChunk

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import OpenAICredentials


class OpenAIInput(InputSchema):
    """Schema for OpenAI chat input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )
    model: str = Field(
        default="gpt-4o",
        description="OpenAI model to use",
    )
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    max_tokens: Optional[int] = Field(
        default=None, description="Maximum tokens in response"
    )
    stream: bool = Field(
        default=False,
        description="Whether to stream the response token by token",
    )


class OpenAIOutput(OutputSchema):
    """Schema for OpenAI chat output"""

    response: str
    used_model: str
    usage: Dict[str, int]


class OpenAIChatSkill(Skill[OpenAIInput, OpenAIOutput]):
    """Skill for interacting with OpenAI models with async support"""

    input_schema = OpenAIInput
    output_schema = OpenAIOutput

    def __init__(self, credentials: Optional[OpenAICredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or OpenAICredentials.from_env()
        self.client = OpenAI(
            api_key=self.credentials.openai_api_key.get_secret_value(),
            organization=self.credentials.openai_organization_id,
        )
        self.async_client = AsyncOpenAI(
            api_key=self.credentials.openai_api_key.get_secret_value(),
            organization=self.credentials.openai_organization_id,
        )

    def _build_messages(self, input_data: OpenAIInput) -> List[Dict[str, str]]:
        """Build messages list from input data including conversation history."""
        messages = [{"role": "system", "content": input_data.system_prompt}]

        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        messages.append({"role": "user", "content": input_data.user_input})
        return messages

    def process_stream(self, input_data: OpenAIInput) -> Generator[str, None, None]:
        """Process the input and stream the response token by token."""
        try:
            messages = self._build_messages(input_data)

            stream = self.client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
                stream=True,
            )

            for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content

        except Exception as e:
            raise ProcessingError(f"OpenAI streaming failed: {str(e)}")

    def process(self, input_data: OpenAIInput) -> OpenAIOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                # For streaming, collect the entire response
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response = "".join(response_chunks)
            else:
                # For non-streaming, use regular completion
                messages = self._build_messages(input_data)
                completion = self.client.chat.completions.create(
                    model=input_data.model,
                    messages=messages,
                    temperature=input_data.temperature,
                    max_tokens=input_data.max_tokens,
                    stream=False,
                )
                response = completion.choices[0].message.content

            return OpenAIOutput(
                response=response,
                used_model=input_data.model,
                usage={
                    "total_tokens": completion.usage.total_tokens,
                    "prompt_tokens": completion.usage.prompt_tokens,
                    "completion_tokens": completion.usage.completion_tokens,
                },
            )

        except Exception as e:
            raise ProcessingError(f"OpenAI chat failed: {str(e)}")

    async def process_async(self, input_data: OpenAIInput) -> OpenAIOutput:
        """Async version of process method"""
        try:
            messages = self._build_messages(input_data)
            completion = await self.async_client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
            )
            return OpenAIOutput(
                response=completion.choices[0].message.content,
                used_model=completion.model,
                usage={
                    "total_tokens": completion.usage.total_tokens,
                    "prompt_tokens": completion.usage.prompt_tokens,
                    "completion_tokens": completion.usage.completion_tokens,
                },
            )
        except Exception as e:
            raise ProcessingError(f"OpenAI async chat failed: {str(e)}")

    async def process_stream_async(
        self, input_data: OpenAIInput
    ) -> AsyncGenerator[str, None]:
        """Async version of stream processor"""
        try:
            messages = self._build_messages(input_data)
            stream = await self.async_client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
                stream=True,
            )
            async for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content
        except Exception as e:
            raise ProcessingError(f"OpenAI async streaming failed: {str(e)}")


ResponseT = TypeVar("ResponseT", bound=BaseModel)


class OpenAIParserInput(InputSchema):
    """Schema for OpenAI structured output input"""

    user_input: str
    system_prompt: str = "You are a helpful assistant that provides structured data."
    model: str = "gpt-4o"
    temperature: float = 0.7
    max_tokens: Optional[int] = None
    response_model: Type[ResponseT]

    class Config:
        arbitrary_types_allowed = True


class OpenAIParserOutput(OutputSchema):
    """Schema for OpenAI structured output"""

    parsed_response: BaseModel
    used_model: str
    tokens_used: int


class OpenAIParserSkill(Skill[OpenAIParserInput, OpenAIParserOutput]):
    """Skill for getting structured responses from OpenAI"""

    input_schema = OpenAIParserInput
    output_schema = OpenAIParserOutput

    def __init__(self, credentials: Optional[OpenAICredentials] = None):
        """Initialize the skill with optional credentials"""
        super().__init__()
        self.credentials = credentials or OpenAICredentials.from_env()
        self.client = OpenAI(
            api_key=self.credentials.openai_api_key.get_secret_value(),
            organization=self.credentials.openai_organization_id,
        )

    def process(self, input_data: OpenAIParserInput) -> OpenAIParserOutput:
        try:
            # Use parse method instead of create
            completion = self.client.beta.chat.completions.parse(
                model=input_data.model,
                messages=[
                    {"role": "system", "content": input_data.system_prompt},
                    {"role": "user", "content": input_data.user_input},
                ],
                response_format=input_data.response_model,
            )

            if completion.choices[0].message.parsed is None:
                raise ProcessingError("Failed to parse response")

            return OpenAIParserOutput(
                parsed_response=completion.choices[0].message.parsed,
                used_model=completion.model,
                tokens_used=completion.usage.total_tokens,
            )

        except Exception as e:
            raise ProcessingError(f"OpenAI parsing failed: {str(e)}")



from abc import ABC, abstractmethod
from typing import Any, Dict, Optional, Type, Generic, TypeVar
from uuid import UUID, uuid4
from .schemas import InputSchema, OutputSchema

# Generic type variables for input and output schemas
InputT = TypeVar("InputT", bound=InputSchema)
OutputT = TypeVar("OutputT", bound=OutputSchema)


class Skill(ABC, Generic[InputT, OutputT]):
    """
    Abstract base class for all skills in Airtrain.
    Each skill must define input/output schemas and implement core processing logic.
    """

    input_schema: Type[InputT]
    output_schema: Type[OutputT]
    _skill_id: Optional[UUID] = None

    @abstractmethod
    def process(self, input_data: InputT) -> OutputT:
        """
        Process the input and generate output according to defined schemas.

        Args:
            input_data: Validated input conforming to input_schema

        Returns:
            Output conforming to output_schema

        Raises:
            ProcessingError: If processing fails
        """
        pass

    def validate_input(self, input_data: Any) -> None:
        """
        Validate input data before processing.

        Args:
            input_data: Raw input data

        Raises:
            InputValidationError: If validation fails
        """
        if not isinstance(input_data, self.input_schema):
            raise InputValidationError(
                f"Input must be an instance of {self.input_schema.__name__}"
            )
        input_data.validate_all()

    def validate_output(self, output_data: Any) -> None:
        """
        Validate output data after processing.

        Args:
            output_data: Processed output data

        Raises:
            OutputValidationError: If validation fails
        """
        if not isinstance(output_data, self.output_schema):
            raise OutputValidationError(
                f"Output must be an instance of {self.output_schema.__name__}"
            )
        output_data.validate_all()

    def evaluate(self, test_dataset: Optional["Dataset"] = None) -> "EvaluationResult":
        """
        Evaluate skill performance.

        Args:
            test_dataset: Optional dataset for evaluation

        Returns:
            EvaluationResult containing metrics
        """
        if not test_dataset:
            test_dataset = self.get_default_test_dataset()

        results = []
        for test_case in test_dataset:
            try:
                output = self.process(test_case.input)
                results.append(self.compare_output(output, test_case.expected))
            except Exception as e:
                results.append(EvaluationError(str(e)))

        return EvaluationResult(results)

    def get_default_test_dataset(self) -> "Dataset":
        """Get default test dataset for evaluation"""
        raise NotImplementedError("No default test dataset provided")

    def compare_output(self, actual: OutputT, expected: OutputT) -> Dict:
        """
        Compare actual output with expected output

        Args:
            actual: Actual output from processing
            expected: Expected output from test case

        Returns:
            Dictionary containing comparison metrics
        """
        raise NotImplementedError("Output comparison not implemented")

    @property
    def skill_id(self) -> UUID:
        """Unique identifier for the skill"""
        if not self._skill_id:
            self._skill_id = uuid4()
        return self._skill_id


class ProcessingError(Exception):
    """Raised when skill processing fails"""

    pass


class InputValidationError(Exception):
    """Raised when input validation fails"""

    pass


class OutputValidationError(Exception):
    """Raised when output validation fails"""

    pass


class EvaluationError:
    """Represents an error during evaluation"""

    def __init__(self, message: str):
        self.message = message


class EvaluationResult:
    """Contains results from skill evaluation"""

    def __init__(self, results: list):
        self.results = results

    def get_metrics(self) -> Dict:
        """Calculate evaluation metrics"""
        return {
            "total_cases": len(self.results),
            "successful": len(
                [r for r in self.results if not isinstance(r, EvaluationError)]
            ),
            "failed": len([r for r in self.results if isinstance(r, EvaluationError)]),
            "results": self.results,
        }


class Dataset:
    """Represents a test dataset for skill evaluation"""

    def __init__(self, test_cases: list):
        self.test_cases = test_cases

    def __iter__(self):
        return iter(self.test_cases)



from datetime import datetime, timedelta
from typing import Optional
from pydantic import Field, SecretStr, validator
from openai import OpenAI

from airtrain.core.credentials import BaseCredentials, CredentialValidationError


class OpenAICredentials(BaseCredentials):
    """OpenAI API credentials with enhanced validation"""

    openai_api_key: SecretStr = Field(..., description="OpenAI API key")
    openai_organization_id: Optional[str] = Field(
        None, description="OpenAI organization ID", pattern="^org-[A-Za-z0-9]{24}$"
    )

    _required_credentials = {"openai_api_key"}

    @validator("openai_api_key")
    def validate_api_key_format(cls, v: SecretStr) -> SecretStr:
        key = v.get_secret_value()
        if not key.startswith("sk-"):
            raise ValueError("OpenAI API key must start with 'sk-'")
        if len(key) < 40:
            raise ValueError("OpenAI API key appears to be too short")
        return v

    async def validate_credentials(self) -> bool:
        """Validate credentials by making a test API call"""
        try:
            client = OpenAI(
                api_key=self.openai_api_key.get_secret_value(),
                organization=self.openai_organization_id,
            )
            # Make minimal API call to validate
            await client.models.list(limit=1)
            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid OpenAI credentials: {str(e)}")



from pydantic import Field, SecretStr
from airtrain.core.credentials import BaseCredentials, CredentialValidationError
import google.genai as genai
from google.cloud import storage
import os

# from google.cloud import storage


class GoogleCloudCredentials(BaseCredentials):
    """Google Cloud credentials"""

    project_id: str = Field(..., description="Google Cloud Project ID")
    service_account_key: SecretStr = Field(..., description="Service Account Key JSON")

    _required_credentials = {"project_id", "service_account_key"}

    async def validate_credentials(self) -> bool:
        """Validate Google Cloud credentials"""
        try:
            # Initialize with service account key
            storage_client = storage.Client.from_service_account_info(
                self.service_account_key.get_secret_value()
            )
            # Test API call
            storage_client.list_buckets(max_results=1)
            return True
        except Exception as e:
            raise CredentialValidationError(
                f"Invalid Google Cloud credentials: {str(e)}"
            )


class GeminiCredentials(BaseCredentials):
    """Gemini API credentials"""

    gemini_api_key: SecretStr = Field(..., description="Gemini API Key")

    _required_credentials = {"gemini_api_key"}

    @classmethod
    def from_env(cls) -> "GeminiCredentials":
        """Create credentials from environment variables"""
        return cls(gemini_api_key=SecretStr(os.environ.get("GEMINI_API_KEY", "")))

    async def validate_credentials(self) -> bool:
        """Validate Gemini API credentials"""
        try:
            # Configure Gemini with API key
            genai.configure(api_key=self.gemini_api_key.get_secret_value())

            # Test API call with a simple model
            model = genai.GenerativeModel("gemini-1.5-flash")
            response = model.generate_content("test")

            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid Gemini credentials: {str(e)}")



from typing import List, Optional, Dict, Any, Generator
from pydantic import Field
from anthropic import Anthropic
import base64
from pathlib import Path
from loguru import logger

from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import AnthropicCredentials


class AnthropicInput(InputSchema):
    """Schema for Anthropic chat input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )
    model: str = Field(
        default="claude-3-opus-20240229", description="Anthropic model to use"
    )
    max_tokens: Optional[int] = Field(
        default=1024, description="Maximum tokens in response"
    )
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    images: List[Path] = Field(
        default_factory=list,
        description="List of image paths to include in the message",
    )
    stream: bool = Field(
        default=False, description="Whether to stream the response progressively"
    )


class AnthropicOutput(OutputSchema):
    """Schema for Anthropic chat output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(
        default_factory=dict, description="Usage statistics from the API"
    )


class AnthropicChatSkill(Skill[AnthropicInput, AnthropicOutput]):
    """Skill for Anthropic chat"""

    input_schema = AnthropicInput
    output_schema = AnthropicOutput

    def __init__(self, credentials: Optional[AnthropicCredentials] = None):
        super().__init__()
        self.credentials = credentials or AnthropicCredentials.from_env()
        self.client = Anthropic(
            api_key=self.credentials.anthropic_api_key.get_secret_value()
        )

    def _build_messages(self, input_data: AnthropicInput) -> List[Dict[str, Any]]:
        """
        Build messages list from input data including conversation history.

        Args:
            input_data: The input data containing system prompt, conversation history, and user input

        Returns:
            List[Dict[str, Any]]: List of messages in the format required by Anthropic
        """
        messages = []

        # Add conversation history if present
        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        # Prepare user message content
        user_message = {"type": "text", "text": input_data.user_input}

        # Add images if present
        if input_data.images:
            content = []
            for image_path in input_data.images:
                with open(image_path, "rb") as img_file:
                    base64_image = base64.b64encode(img_file.read()).decode("utf-8")
                content.append(
                    {
                        "type": "image",
                        "source": {
                            "type": "base64",
                            "media_type": "image/jpeg",
                            "data": base64_image,
                        },
                    }
                )
            content.append(user_message)
            messages.append({"role": "user", "content": content})
        else:
            messages.append({"role": "user", "content": [user_message]})

        return messages

    def process_stream(self, input_data: AnthropicInput) -> Generator[str, None, None]:
        """Process the input and stream the response token by token."""
        try:
            messages = self._build_messages(input_data)

            with self.client.beta.messages.stream(
                model=input_data.model,
                system=input_data.system_prompt,
                messages=messages,
                max_tokens=input_data.max_tokens,
                temperature=input_data.temperature,
            ) as stream:
                for chunk in stream.text_stream:
                    yield chunk

        except Exception as e:
            logger.exception(f"Anthropic streaming failed: {str(e)}")
            raise ProcessingError(f"Anthropic streaming failed: {str(e)}")

    def process(self, input_data: AnthropicInput) -> AnthropicOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response = "".join(response_chunks)
                usage = {}  # Usage stats not available in streaming
            else:
                messages = self._build_messages(input_data)
                response = self.client.messages.create(
                    model=input_data.model,
                    system=input_data.system_prompt,
                    messages=messages,
                    max_tokens=input_data.max_tokens,
                    temperature=input_data.temperature,
                )
                usage = response.usage.model_dump() if response.usage else {}

            return AnthropicOutput(
                response=response.content[0].text,
                used_model=input_data.model,
                usage=usage,
            )

        except Exception as e:
            logger.exception(f"Anthropic processing failed: {str(e)}")
            raise ProcessingError(f"Anthropic processing failed: {str(e)}")



from setuptools import setup, find_packages  # type: ignore
import os
import sys
import re


# Read version from __init__.py
def get_version():
    init_path = os.path.join(os.path.dirname(__file__), "airtrain", "__init__.py")
    with open(init_path, "r", encoding="utf-8") as f:
        version_file = f.read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def get_changelog() -> str:
    """Read the changelog content from absolute path."""
    # Get the absolute path to the setup.py directory
    setup_dir = os.path.dirname(os.path.abspath(__file__))
    changelog_path = os.path.abspath(os.path.join(setup_dir, "changelog.md"))

    with open(changelog_path, "r", encoding="utf-8") as f:
        # Skip the frontmatter and title
        lines = f.readlines()
        changelog_content = []
        skip_frontmatter = False
        for line in lines:
            if line.strip() == "---":
                skip_frontmatter = not skip_frontmatter
                continue
            if not skip_frontmatter and not line.startswith("# Changelog"):
                changelog_content.append(line)
    return "".join(changelog_content)


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Combine README and Changelog
full_description = f"{long_description}\n\n## Changelog\n{get_changelog()}"

setup(
    name="airtrain",
    version=get_version(),
    author="Dheeraj Pai",
    author_email="helloworldcmu@gmail.com",
    description="A platform for building and deploying AI agents with structured skills",
    long_description=full_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rosaboyle/airtrain.dev",
    packages=find_packages(include=["airtrain", "airtrain.*"]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    python_requires=">=3.8",
    install_requires=[
        "pydantic>=2.10.6",
        "openai>=1.60.1",
        "python-dotenv>=1.0.1",
        "PyYAML>=6.0.2",
        "firebase-admin>=6.6.0",  # Optional, only if using Firebase
        "loguru>=0.7.3",  # For logging
        "requests>=2.32.3",
        "boto3>=1.36.6",  # For AWS services
        "together>=1.3.13",  # For Together AI integration
        "anthropic>=0.45.0",  # For Anthropic AI integration
        "groq>=0.15.0",  # For Groq AI integration
        "cerebras-cloud-sdk>=1.19.0",
        "google-genai>=1.0.0",
        "fireworks-ai>=0.15.12",
        "google-generativeai>=0.8.4",
        "click>=8.0.0",  # For CLI support
        "rich>=13.3.1",  # For beautiful terminal output
        "prompt-toolkit>=3.0.36",  # For interactive prompts
        "colorama>=0.4.6",  # For colored terminal text
        "typer>=0.9.0",  # For building CLI applications
    ],
    extras_require={
        "dev": [
            "black>=24.10.0",
            "flake8>=7.1.1",
            "isort>=5.13.0",
            "mypy>=1.9.0",
            "pytest>=7.0.0",
            "twine>=4.0.0",
            "build>=0.10.0",
            "types-PyYAML>=6.0",
            "types-requests>=2.31.0",
            "types-Markdown>=3.5.0",
        ]
    },
    entry_points={
        "console_scripts": [
            "airtrain=airtrain.cli.main:main",
        ],
    },
)



from pydantic import Field, SecretStr
from airtrain.core.credentials import BaseCredentials, CredentialValidationError


class CerebrasCredentials(BaseCredentials):
    """Cerebras credentials"""

    cerebras_api_key: SecretStr = Field(..., description="Cerebras API key")

    _required_credentials = {"cerebras_api_key"}

    async def validate_credentials(self) -> bool:
        """Validate Cerebras credentials"""
        try:
            # Implement Cerebras-specific validation
            # This would depend on their API client implementation
            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid Cerebras credentials: {str(e)}")



from typing import Generator, Optional, Dict, Any, List
from pydantic import Field
from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import GroqCredentials
from groq import Groq


class GroqInput(InputSchema):
    """Schema for Groq input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )
    model: str = Field(
        default="deepseek-r1-distill-llama-70b-specdec", description="Groq model to use"
    )
    max_tokens: int = Field(default=1024, description="Maximum tokens in response")
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    stream: bool = Field(
        default=False, description="Whether to stream the response progressively"
    )


class GroqOutput(OutputSchema):
    """Schema for Groq output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(
        default_factory=dict, description="Usage statistics from the API"
    )


class GroqChatSkill(Skill[GroqInput, GroqOutput]):
    """Skill for Groq chat"""

    input_schema = GroqInput
    output_schema = GroqOutput

    def __init__(self, credentials: Optional[GroqCredentials] = None):
        super().__init__()
        self.credentials = credentials or GroqCredentials.from_env()
        self.client = Groq(api_key=self.credentials.groq_api_key.get_secret_value())

    def _build_messages(self, input_data: GroqInput) -> List[Dict[str, str]]:
        """
        Build messages list from input data including conversation history.

        Args:
            input_data: The input data containing system prompt, conversation history, and user input

        Returns:
            List[Dict[str, str]]: List of messages in the format required by Groq
        """
        messages = [{"role": "system", "content": input_data.system_prompt}]

        # Add conversation history if present
        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        # Add current user input
        messages.append({"role": "user", "content": input_data.user_input})

        return messages

    def process_stream(self, input_data: GroqInput) -> Generator[str, None, None]:
        """Process the input and stream the response token by token."""
        try:
            messages = self._build_messages(input_data)

            stream = self.client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
                stream=True,
            )

            for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content

        except Exception as e:
            raise ProcessingError(f"Groq streaming failed: {str(e)}")

    def process(self, input_data: GroqInput) -> GroqOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response = "".join(response_chunks)
                usage = {}  # Usage stats not available in streaming
            else:
                messages = self._build_messages(input_data)
                completion = self.client.chat.completions.create(
                    model=input_data.model,
                    messages=messages,
                    temperature=input_data.temperature,
                    max_tokens=input_data.max_tokens,
                    stream=False,
                )
                response = completion.choices[0].message.content
                usage = {
                    "total_tokens": completion.usage.total_tokens,
                    "prompt_tokens": completion.usage.prompt_tokens,
                    "completion_tokens": completion.usage.completion_tokens,
                }

            return GroqOutput(
                response=response, used_model=input_data.model, usage=usage
            )

        except Exception as e:
            raise ProcessingError(f"Groq processing failed: {str(e)}")



from typing import Optional, TypeVar
from pydantic import Field
from .skills import OpenAIChatSkill, OpenAIInput, OpenAIOutput
from .credentials import OpenAICredentials

T = TypeVar("T", bound=OpenAIInput)


class ChineseAssistantInput(OpenAIInput):
    """Schema for Chinese Assistant input"""

    user_input: str = Field(
        ..., description="User's input text (can be in any language)"
    )
    system_prompt: str = Field(
        default="你是一个有帮助的助手。请用中文回答所有问题，即使问题是用其他语言问的。回答要准确、礼貌、专业。",
        description="System prompt in Chinese",
    )
    model: str = Field(default="gpt-4o", description="OpenAI model to use")
    max_tokens: int = Field(default=8096, description="Maximum tokens in response")
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )


class ChineseAssistantSkill(OpenAIChatSkill):
    """Skill for Chinese language assistance"""

    input_schema = ChineseAssistantInput
    output_schema = OpenAIOutput

    def __init__(self, credentials: Optional[OpenAICredentials] = None):
        super().__init__(credentials)

    def process(self, input_data: T) -> OpenAIOutput:
        # Add language check to ensure response is in Chinese
        if "你是" not in input_data.system_prompt:
            input_data.system_prompt = (
                "你是一个中文助手。" + input_data.system_prompt + "请用中文回答。"
            )

        return super().process(input_data)



from typing import Dict, List, Optional, Set
import os
import json
from pathlib import Path
from abc import ABC, abstractmethod
import dotenv
from pydantic import BaseModel, Field, SecretStr
import yaml


class CredentialError(Exception):
    """Base exception for credential-related errors"""

    pass


class CredentialNotFoundError(CredentialError):
    """Raised when a required credential is not found"""

    pass


class CredentialValidationError(CredentialError):
    """Raised when credentials fail validation"""

    pass


class BaseCredentials(BaseModel):
    """Base class for all credential configurations"""

    _loaded: bool = False
    _required_credentials: Set[str] = set()

    def load_to_env(self) -> None:
        """Load credentials into environment variables"""
        for field_name, field_value in self:
            if isinstance(field_value, SecretStr):
                value = field_value.get_secret_value()
            else:
                value = str(field_value)
            os.environ[field_name.upper()] = value
        self._loaded = True

    @classmethod
    def from_env(cls) -> "BaseCredentials":
        """Create credentials instance from environment variables"""
        field_values = {}
        for field_name in cls.model_fields:
            env_key = field_name.upper()
            if env_value := os.getenv(env_key):
                field_values[field_name] = env_value
        return cls(**field_values)

    @classmethod
    def from_file(cls, file_path: Path) -> "BaseCredentials":
        """Load credentials from a file (supports .env, .json, .yaml)"""
        if not file_path.exists():
            raise FileNotFoundError(f"Credentials file not found: {file_path}")

        if file_path.suffix == ".env":
            dotenv.load_dotenv(file_path)
            return cls.from_env()

        elif file_path.suffix == ".json":
            with open(file_path) as f:
                data = json.load(f)
            return cls(**data)

        elif file_path.suffix in {".yaml", ".yml"}:
            with open(file_path) as f:
                data = yaml.safe_load(f)
            return cls(**data)

        else:
            raise ValueError(f"Unsupported file format: {file_path.suffix}")

    def save_to_file(self, file_path: Path) -> None:
        """Save credentials to a file"""
        data = self.model_dump(exclude={"_loaded"})

        # Convert SecretStr to plain strings for saving
        for key, value in data.items():
            if isinstance(value, SecretStr):
                data[key] = value.get_secret_value()

        if file_path.suffix == ".env":
            with open(file_path, "w") as f:
                for key, value in data.items():
                    f.write(f"{key.upper()}={value}\n")

        elif file_path.suffix == ".json":
            with open(file_path, "w") as f:
                json.dump(data, f, indent=2)

        elif file_path.suffix in {".yaml", ".yml"}:
            with open(file_path, "w") as f:
                yaml.dump(data, f)

        else:
            raise ValueError(f"Unsupported file format: {file_path.suffix}")

    async def validate_credentials(self) -> bool:
        """Validate that all required credentials are present"""
        missing = []
        for field_name in self._required_credentials:
            value = getattr(self, field_name, None)
            if value is None or (
                isinstance(value, SecretStr) and not value.get_secret_value()
            ):
                missing.append(field_name)

        if missing:
            raise CredentialValidationError(
                f"Missing required credentials: {', '.join(missing)}"
            )
        return True

    def clear_from_env(self) -> None:
        """Remove credentials from environment variables"""
        for field_name in self.model_fields:
            env_key = field_name.upper()
            if env_key in os.environ:
                del os.environ[env_key]
        self._loaded = False



"""Fireworks AI integration module"""

from .credentials import FireworksCredentials
from .skills import FireworksChatSkill, FireworksInput, FireworksOutput

__all__ = [
    "FireworksCredentials",
    "FireworksChatSkill",
    "FireworksInput",
    "FireworksOutput",
]



from pydantic import Field
from airtrain.core.credentials import BaseCredentials, CredentialValidationError
from importlib.util import find_spec


class OllamaCredentials(BaseCredentials):
    """Ollama credentials"""

    host: str = Field(default="http://localhost:11434", description="Ollama host URL")
    timeout: int = Field(default=30, description="Request timeout in seconds")

    async def validate_credentials(self) -> bool:
        """Validate Ollama credentials"""
        if find_spec("ollama") is None:
            raise CredentialValidationError(
                "Ollama package is not installed. Please install it using: pip install ollama"
            )

        try:
            from ollama import Client

            client = Client(host=self.host)
            await client.list()
            return True
        except Exception as e:
            raise CredentialValidationError(f"Invalid Ollama connection: {str(e)}")



from typing import Dict, NamedTuple


class AudioModelConfig(NamedTuple):
    organization: str
    display_name: str


TOGETHER_AUDIO_MODELS: Dict[str, AudioModelConfig] = {
    "Cartesia/Sonic": AudioModelConfig(
        organization="Cartesia", display_name="Cartesia/Sonic"
    )
}


def get_audio_model_config(model_id: str) -> AudioModelConfig:
    """Get audio model configuration by model ID"""
    if model_id not in TOGETHER_AUDIO_MODELS:
        raise ValueError(f"Model {model_id} not found in Together AI audio models")
    return TOGETHER_AUDIO_MODELS[model_id]


def list_audio_models_by_organization(organization: str) -> Dict[str, AudioModelConfig]:
    """Get all audio models for a specific organization"""
    return {
        model_id: config
        for model_id, config in TOGETHER_AUDIO_MODELS.items()
        if config.organization.lower() == organization.lower()
    }


def get_default_audio_model() -> str:
    """Get the default audio model ID"""
    return "Cartesia/Sonic"



from typing import Optional, Dict, Any, List, Generator
from pydantic import Field
from airtrain.core.skills import Skill, ProcessingError
from airtrain.core.schemas import InputSchema, OutputSchema
from .credentials import TogetherAICredentials
from .models import TogetherAIImageInput, TogetherAIImageOutput, GeneratedImage
from pathlib import Path
import base64
import time
from together import Together


class TogetherAIInput(InputSchema):
    """Schema for Together AI input"""

    user_input: str = Field(..., description="User's input text")
    system_prompt: str = Field(
        default="You are a helpful assistant.",
        description="System prompt to guide the model's behavior",
    )
    conversation_history: List[Dict[str, str]] = Field(
        default_factory=list,
        description="List of previous conversation messages in [{'role': 'user|assistant', 'content': 'message'}] format",
    )
    model: str = Field(
        default="deepseek-ai/DeepSeek-R1", description="Together AI model to use"
    )
    max_tokens: int = Field(default=1024, description="Maximum tokens in response")
    temperature: float = Field(
        default=0.7, description="Temperature for response generation", ge=0, le=1
    )
    stream: bool = Field(default=False, description="Whether to stream the response")


class TogetherAIOutput(OutputSchema):
    """Schema for Together AI output"""

    response: str = Field(..., description="Model's response text")
    used_model: str = Field(..., description="Model used for generation")
    usage: Dict[str, Any] = Field(default_factory=dict, description="Usage statistics")


class TogetherAIChatSkill(Skill[TogetherAIInput, TogetherAIOutput]):
    """Skill for Together AI chat"""

    input_schema = TogetherAIInput
    output_schema = TogetherAIOutput

    def __init__(self, credentials: Optional[TogetherAICredentials] = None):
        super().__init__()
        self.credentials = credentials or TogetherAICredentials.from_env()
        self.client = Together(
            api_key=self.credentials.together_api_key.get_secret_value()
        )

    def _build_messages(self, input_data: TogetherAIInput) -> List[Dict[str, str]]:
        """
        Build messages list from input data including conversation history.

        Args:
            input_data: The input data containing system prompt, conversation history, and user input

        Returns:
            List[Dict[str, str]]: List of messages in the format required by Together AI
        """
        messages = [{"role": "system", "content": input_data.system_prompt}]

        # Add conversation history if present
        if input_data.conversation_history:
            messages.extend(input_data.conversation_history)

        # Add current user input
        messages.append({"role": "user", "content": input_data.user_input})

        return messages

    def process_stream(self, input_data: TogetherAIInput) -> Generator[str, None, None]:
        """Process the input and stream the response token by token."""
        try:
            messages = self._build_messages(input_data)

            stream = self.client.chat.completions.create(
                model=input_data.model,
                messages=messages,
                temperature=input_data.temperature,
                max_tokens=input_data.max_tokens,
                stream=True,
            )

            for chunk in stream:
                if chunk.choices[0].delta.content is not None:
                    yield chunk.choices[0].delta.content

        except Exception as e:
            raise ProcessingError(f"Together AI streaming failed: {str(e)}")

    def process(self, input_data: TogetherAIInput) -> TogetherAIOutput:
        """Process the input and return the complete response."""
        try:
            if input_data.stream:
                response_chunks = []
                for chunk in self.process_stream(input_data):
                    response_chunks.append(chunk)
                response = "".join(response_chunks)
            else:
                messages = self._build_messages(input_data)
                completion = self.client.chat.completions.create(
                    model=input_data.model,
                    messages=messages,
                    temperature=input_data.temperature,
                    max_tokens=input_data.max_tokens,
                    stream=False,
                )
                response = completion.choices[0].message.content

            return TogetherAIOutput(
                response=response,
                used_model=input_data.model,
                usage=(
                    completion.usage.model_dump()
                    if hasattr(completion, "usage")
                    else {}
                ),
            )

        except Exception as e:
            raise ProcessingError(f"Together AI processing failed: {str(e)}")


class TogetherAIImageSkill(Skill[TogetherAIImageInput, TogetherAIImageOutput]):
    """Skill for Together AI image generation"""

    input_schema = TogetherAIImageInput
    output_schema = TogetherAIImageOutput

    def __init__(self, credentials: Optional[TogetherAICredentials] = None):
        super().__init__()
        self.credentials = credentials or TogetherAICredentials.from_env()
        self.client = Together(
            api_key=self.credentials.together_api_key.get_secret_value()
        )

    def process(self, input_data: TogetherAIImageInput) -> TogetherAIImageOutput:
        try:
            start_time = time.time()

            # Generate images
            response = self.client.images.generate(
                prompt=input_data.prompt,
                model=input_data.model,
                steps=input_data.steps,
                n=input_data.n,
                size=input_data.size,
                negative_prompt=input_data.negative_prompt,
                seed=input_data.seed,
            )

            # Calculate total time
            total_time = time.time() - start_time

            # Convert response to our output format
            generated_images = [
                GeneratedImage(
                    b64_json=img.b64_json,
                    seed=getattr(img, "seed", None),
                    finish_reason=getattr(img, "finish_reason", None),
                )
                for img in response.data
            ]

            return TogetherAIImageOutput(
                images=generated_images,
                model=input_data.model,
                prompt=input_data.prompt,
                total_time=total_time,
                usage=getattr(response, "usage", {}),
            )

        except Exception as e:
            raise ProcessingError(f"Together AI image generation failed: {str(e)}")

    def save_images(
        self, output: TogetherAIImageOutput, output_dir: Path
    ) -> List[Path]:
        """
        Save generated images to disk

        Args:
            output (TogetherAIImageOutput): Generation output containing images
            output_dir (Path): Directory to save images

        Returns:
            List[Path]: List of paths to saved images
        """
        output_dir = Path(output_dir)
        output_dir.mkdir(parents=True, exist_ok=True)

        saved_paths = []
        for i, img in enumerate(output.images):
            output_path = output_dir / f"image_{i}.png"
            image_data = base64.b64decode(img.b64_json)

            with open(output_path, "wb") as f:
                f.write(image_data)

            saved_paths.append(output_path)

        return saved_paths
